@@ -88,7 +88,7 @@ pub(crate) struct Guard(
 fn locking_global() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_packages(&paths).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
 
@@ -96,7 +96,7 @@ fn locking_global() {
 fn locking_dev_erlang() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_target(&paths, Mode::Dev, Target::Erlang).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
 
@@ -104,7 +104,7 @@ fn locking_dev_erlang() {
 fn locking_prod_erlang() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_target(&paths, Mode::Prod, Target::Erlang).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
 
@@ -112,7 +112,7 @@ fn locking_prod_erlang() {
 fn locking_lsp_erlang() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_target(&paths, Mode::Lsp, Target::Erlang).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
 
@@ -120,7 +120,7 @@ fn locking_lsp_erlang() {
 fn locking_dev_javascript() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_target(&paths, Mode::Dev, Target::JavaScript).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
 
@@ -128,7 +128,7 @@ fn locking_dev_javascript() {
 fn locking_prod_javascript() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_target(&paths, Mode::Prod, Target::JavaScript).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
 
@@ -136,6 +136,6 @@ fn locking_prod_javascript() {
 fn locking_lsp_javascript() {
     let paths = crate::project_paths_at_current_directory_without_toml();
     let lock = BuildLock::new_target(&paths, Mode::Lsp, Target::JavaScript).expect("make lock");
-    let _guard1 = lock.lock(&gleam_core::build::NullTelemetry);
+    let _guard1: Guard = lock.lock(&gleam_core::build::NullTelemetry).unwrap();
     println!("Locked!")
 }
