@@ -22,13 +22,13 @@ use crate::{
     parse::SpannedString,
     reference::{EntityKind, ReferenceKind},
     type_::{
-        self, AccessorsMap, Deprecation, ModuleInterface, Opaque, PatternConstructor,
+        self, AccessorsMap, Deprecation, FieldMap, ModuleInterface, Opaque, PatternConstructor,
         RecordAccessor, References, Type, TypeAliasConstructor, TypeConstructor,
         TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
         ValueConstructorVariant, Warning,
         environment::*,
         error::{Error, FeatureKind, MissingAnnotation, Named, Problems, convert_unify_error},
-        expression::{ExprTyper, FunctionDefinition, Implementations},
+        expression::{ExprTyper, FunctionDefinition, Implementations, Purity},
         fields::FieldMapBuilder,
         hydrator::Hydrator,
         prelude::*,
@@ -88,6 +88,13 @@ impl Inferred<PatternConstructor> {
             Inferred::Unknown => None,
         }
     }
+
+    pub fn field_map(&self) -> Option<&FieldMap> {
+        match self {
+            Inferred::Known(value) => value.field_map.as_ref(),
+            Inferred::Unknown => None,
+        }
+    }
 }
 
 /// How the compiler should treat target support.
@@ -200,18 +207,21 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         }
 
         let documentation = std::mem::take(&mut module.documentation);
-        let env = Environment::new(
-            self.ids.clone(),
-            self.package_config.name.clone(),
-            self.package_config
+        let env = EnvironmentArguments {
+            ids: self.ids.clone(),
+            current_package: self.package_config.name.clone(),
+            gleam_version: self
+                .package_config
                 .gleam_version
                 .clone()
                 .map(|version| version.as_pubgrub()),
-            self.module_name.clone(),
-            self.target,
-            self.importable_modules,
-            self.target_support,
-        );
+            current_module: self.module_name.clone(),
+            target: self.target,
+            importable_modules: self.importable_modules,
+            target_support: self.target_support,
+            current_origin: self.origin,
+        }
+        .build();
 
         let statements = GroupedStatements::new(module.into_iter_statements(self.target));
         let statements_count = statements.len();
@@ -238,9 +248,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         }
 
         for f in &statements.functions {
-            if let Err(error) = self.register_value_from_function(f, &mut env) {
-                return self.all_errors(error);
-            }
+            self.register_value_from_function(f, &mut env);
         }
 
         // Infer the types of each statement in the module
@@ -485,6 +493,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             external_javascript,
             return_type: (),
             implementations: _,
+            purity: _,
         } = f;
         let (name_location, name) = name.expect("Function in a definition must be named");
         let target = environment.target;
@@ -560,6 +569,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 body,
                 expr_typer.implementations,
                 expr_typer.minimum_required_version,
+                expr_typer.purity,
             ))
         });
 
@@ -567,9 +577,9 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         // function then register the error and continue anaylsis using the best
         // information that we have, so we can still learn about the rest of the
         // module.
-        let (type_, body, implementations, required_version) = match result {
-            Ok((type_, body, implementations, required_version)) => {
-                (type_, body, implementations, required_version)
+        let (type_, body, implementations, required_version, purity) = match result {
+            Ok((type_, body, implementations, required_version, purity)) => {
+                (type_, body, implementations, required_version, purity)
             }
             Err(error) => {
                 self.problems.error(error);
@@ -582,7 +592,13 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                     },
                 }));
                 let implementations = Implementations::supporting_all();
-                (type_, body, implementations, Version::new(1, 0, 0))
+                (
+                    type_,
+                    body,
+                    implementations,
+                    Version::new(1, 0, 0),
+                    Purity::Impure,
+                )
             }
         };
 
@@ -648,6 +664,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             arity: typed_args.len(),
             location,
             implementations,
+            purity,
         };
 
         environment.insert_variable(
@@ -682,6 +699,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             external_erlang,
             external_javascript,
             implementations,
+            purity,
         })
     }
 
@@ -920,7 +938,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             .collect();
         let typed_parameters = environment
             .get_type_constructor(&None, &name)
-            .expect("Could not find preregistered type constructor ")
+            .expect("Could not find preregistered type constructor")
             .parameters
             .clone();
 
@@ -1383,7 +1401,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         &mut self,
         f: &UntypedFunction,
         environment: &mut Environment<'_>,
-    ) -> Result<(), Error> {
+    ) {
         let Function {
             name,
             arguments: args,
@@ -1398,6 +1416,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             body: _,
             return_type: _,
             implementations,
+            purity,
         } = f;
         let (name_location, name) = name.as_ref().expect("A module's function must be named");
 
@@ -1417,7 +1436,9 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         {
             check_argument_names(names, &mut self.problems);
 
-            builder.add(names.get_label(), *location)?;
+            if let Err(error) = builder.add(names.get_label(), *location) {
+                self.problems.error(error);
+            }
         }
         let field_map = builder.finish();
         let mut hydrator = Hydrator::new();
@@ -1429,11 +1450,30 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         let arg_types = args
             .iter()
             .map(|arg| {
-                hydrator.type_from_option_ast(&arg.annotation, environment, &mut self.problems)
+                match hydrator.type_from_option_ast(
+                    &arg.annotation,
+                    environment,
+                    &mut self.problems,
+                ) {
+                    Ok(type_) => type_,
+                    Err(error) => {
+                        self.problems.error(error);
+                        environment.new_unbound_var()
+                    }
+                }
             })
-            .try_collect()?;
+            .collect();
+
         let return_type =
-            hydrator.type_from_option_ast(return_annotation, environment, &mut self.problems)?;
+            match hydrator.type_from_option_ast(return_annotation, environment, &mut self.problems)
+            {
+                Ok(type_) => type_,
+                Err(error) => {
+                    self.problems.error(error);
+                    environment.new_unbound_var()
+                }
+            };
+
         let type_ = fn_(arg_types, return_type);
         let _ = self.hydrators.insert(name.clone(), hydrator);
 
@@ -1451,6 +1491,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             arity: args.len(),
             location: *location,
             implementations: *implementations,
+            purity: *purity,
         };
         environment.insert_variable(
             name.clone(),
@@ -1459,7 +1500,6 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             *publicity,
             deprecation.clone(),
         );
-        Ok(())
     }
 
     fn check_for_type_leaks(&mut self, value: &ValueConstructor) {
@@ -1720,6 +1760,7 @@ fn generalise_function(
         external_erlang,
         external_javascript,
         implementations,
+        purity,
     } = function;
 
     let (name_location, name) = name.expect("Function in a definition must be named");
@@ -1748,6 +1789,7 @@ fn generalise_function(
         arity: args.len(),
         location,
         implementations,
+        purity,
     };
     environment.insert_variable(
         name.clone(),
@@ -1780,6 +1822,7 @@ fn generalise_function(
         external_erlang,
         external_javascript,
         implementations,
+        purity,
     })
 }
 
