@@ -40,6 +40,7 @@
 
 use crate::{
     analyse::Inferred,
+    exhaustiveness::CompiledCase,
     type_::{
         ModuleValueConstructor, PatternConstructor, TypedCallArg, ValueConstructor,
         error::VariableOrigin,
@@ -54,7 +55,7 @@ use crate::type_::Type;
 
 use super::{
     AssignName, BinOp, BitArrayOption, CallArg, Definition, Pattern, PipelineAssignmentKind,
-    SrcSpan, Statement, TodoKind, TypeAst, TypedArg, TypedAssignment, TypedClause,
+    SrcSpan, Statement, TodoKind, TypeAst, TypedArg, TypedAssert, TypedAssignment, TypedClause,
     TypedClauseGuard, TypedConstant, TypedCustomType, TypedDefinition, TypedExpr,
     TypedExprBitArraySegment, TypedFunction, TypedModule, TypedModuleConstant, TypedPattern,
     TypedPatternBitArraySegment, TypedPipelineAssignment, TypedStatement, TypedUse,
@@ -207,8 +208,9 @@ pub trait Visit<'ast> {
         type_: &'ast Arc<Type>,
         subjects: &'ast [TypedExpr],
         clauses: &'ast [TypedClause],
+        compiled_case: &'ast CompiledCase,
     ) {
-        visit_typed_expr_case(self, location, type_, subjects, clauses);
+        visit_typed_expr_case(self, location, type_, subjects, clauses, compiled_case);
     }
 
     fn visit_typed_expr_record_access(
@@ -328,6 +330,10 @@ pub trait Visit<'ast> {
         visit_typed_use(self, use_);
     }
 
+    fn visit_typed_assert(&mut self, assert: &'ast TypedAssert) {
+        visit_typed_assert(self, assert);
+    }
+
     fn visit_typed_pipeline_assignment(&mut self, assignment: &'ast TypedPipelineAssignment) {
         visit_typed_pipeline_assignment(self, assignment);
     }
@@ -536,6 +542,10 @@ pub trait Visit<'ast> {
         );
     }
 
+    fn visit_typed_pattern_invalid(&mut self, location: &'ast SrcSpan, type_: &'ast Arc<Type>) {
+        visit_typed_pattern_invalid(self, location, type_);
+    }
+
     fn visit_type_ast(&mut self, node: &'ast TypeAst) {
         visit_type_ast(self, node);
     }
@@ -744,6 +754,7 @@ where
             args,
             body,
             return_annotation,
+            purity: _,
         } => v.visit_typed_expr_fn(location, type_, kind, args, body, return_annotation),
         TypedExpr::List {
             location,
@@ -770,7 +781,8 @@ where
             type_,
             subjects,
             clauses,
-        } => v.visit_typed_expr_case(location, type_, subjects, clauses),
+            compiled_case,
+        } => v.visit_typed_expr_case(location, type_, subjects, clauses, compiled_case),
         TypedExpr::RecordAccess {
             location,
             field_start,
@@ -991,6 +1003,7 @@ pub fn visit_typed_expr_case<'a, V>(
     _type_: &'a Arc<Type>,
     subjects: &'a [TypedExpr],
     clauses: &'a [TypedClause],
+    _compiled_case: &'a CompiledCase,
 ) where
     V: Visit<'a> + ?Sized,
 {
@@ -1150,6 +1163,7 @@ where
         Statement::Expression(expr) => v.visit_typed_expr(expr),
         Statement::Assignment(assignment) => v.visit_typed_assignment(assignment),
         Statement::Use(use_) => v.visit_typed_use(use_),
+        Statement::Assert(assert) => v.visit_typed_assert(assert),
     }
 }
 
@@ -1169,6 +1183,16 @@ where
     // TODO: We should also visit the typed patterns!!
 }
 
+pub fn visit_typed_assert<'a, V>(v: &mut V, assert: &'a TypedAssert)
+where
+    V: Visit<'a> + ?Sized,
+{
+    v.visit_typed_expr(&assert.value);
+    if let Some(message) = &assert.message {
+        v.visit_typed_expr(message);
+    }
+}
+
 pub fn visit_typed_call_arg<'a, V>(v: &mut V, arg: &'a TypedCallArg)
 where
     V: Visit<'a> + ?Sized,
@@ -1520,7 +1544,7 @@ where
             left_side_string,
             right_side_assignment,
         ),
-        Pattern::Invalid { location, type_ } => v.visit_typed_expr_invalid(location, type_),
+        Pattern::Invalid { location, type_ } => v.visit_typed_pattern_invalid(location, type_),
     }
 }
 
@@ -1704,6 +1728,12 @@ pub fn visit_typed_pattern_string_prefix<'a, V>(
 {
 }
 
+pub fn visit_typed_pattern_invalid<'a, V>(_v: &mut V, _location: &'a SrcSpan, _type_: &'a Arc<Type>)
+where
+    V: Visit<'a> + ?Sized,
+{
+}
+
 pub fn visit_typed_expr_invalid<'a, V>(_v: &mut V, _location: &'a SrcSpan, _type_: &'a Arc<Type>)
 where
     V: Visit<'a> + ?Sized,
