@@ -5,14 +5,15 @@ use vec1::Vec1;
 use crate::{
     analyse::Inferred,
     ast::{
-        AssignName, Assignment, BinOp, CallArg, Constant, Definition, FunctionLiteralKind, Pattern,
-        RecordBeingUpdated, SrcSpan, Statement, TargetedDefinition, TodoKind, TypeAst,
+        Assert, AssignName, Assignment, BinOp, CallArg, Constant, Definition, FunctionLiteralKind,
+        Pattern, RecordBeingUpdated, SrcSpan, Statement, TargetedDefinition, TodoKind, TypeAst,
         TypeAstConstructor, TypeAstFn, TypeAstHole, TypeAstTuple, TypeAstVar, UntypedArg,
-        UntypedAssignment, UntypedClause, UntypedConstant, UntypedConstantBitArraySegment,
-        UntypedCustomType, UntypedDefinition, UntypedExpr, UntypedExprBitArraySegment,
-        UntypedFunction, UntypedImport, UntypedModule, UntypedModuleConstant, UntypedPattern,
-        UntypedPatternBitArraySegment, UntypedRecordUpdateArg, UntypedStatement, UntypedTypeAlias,
-        UntypedUse, UntypedUseAssignment, Use, UseAssignment,
+        UntypedAssert, UntypedAssignment, UntypedClause, UntypedConstant,
+        UntypedConstantBitArraySegment, UntypedCustomType, UntypedDefinition, UntypedExpr,
+        UntypedExprBitArraySegment, UntypedFunction, UntypedImport, UntypedModule,
+        UntypedModuleConstant, UntypedPattern, UntypedPatternBitArraySegment,
+        UntypedRecordUpdateArg, UntypedStatement, UntypedTypeAlias, UntypedUse,
+        UntypedUseAssignment, Use, UseAssignment,
     },
     build::Target,
     type_::error::VariableOrigin,
@@ -606,9 +607,10 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
         match statement {
             Statement::Expression(expression) => Statement::Expression(expression),
             Statement::Assignment(assignment) => {
-                Statement::Assignment(self.fold_assignment(assignment))
+                Statement::Assignment(Box::new(self.fold_assignment(*assignment)))
             }
             Statement::Use(use_) => Statement::Use(self.fold_use(use_)),
+            Statement::Assert(assert) => Statement::Assert(self.fold_assert(assert)),
         }
     }
 
@@ -617,23 +619,27 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
         match statement {
             Statement::Expression(expression) => Statement::Expression(self.fold_expr(expression)),
 
-            Statement::Assignment(Assignment {
-                location,
-                value,
-                pattern,
-                kind,
-                annotation,
-            }) => {
-                let pattern = self.fold_pattern(pattern);
-                let annotation = annotation.map(|type_| self.fold_type(type_));
-                let value = Box::new(self.fold_expr(*value));
-                Statement::Assignment(Assignment {
+            Statement::Assignment(assignment) => {
+                let Assignment {
                     location,
                     value,
                     pattern,
                     kind,
                     annotation,
-                })
+                    compiled_case,
+                } = *assignment;
+
+                let pattern = self.fold_pattern(pattern);
+                let annotation = annotation.map(|type_| self.fold_type(type_));
+                let value = self.fold_expr(value);
+                Statement::Assignment(Box::new(Assignment {
+                    location,
+                    value,
+                    pattern,
+                    kind,
+                    annotation,
+                    compiled_case,
+                }))
             }
 
             Statement::Use(Use {
@@ -660,6 +666,20 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
                     assignments,
                 })
             }
+
+            Statement::Assert(Assert {
+                location,
+                value,
+                message,
+            }) => {
+                let value = self.fold_expr(value);
+                let message = message.map(|message| self.fold_expr(message));
+                Statement::Assert(Assert {
+                    location,
+                    value,
+                    message,
+                })
+            }
         }
     }
 
@@ -886,6 +906,10 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
     fn fold_use(&mut self, use_: UntypedUse) -> UntypedUse {
         use_
     }
+
+    fn fold_assert(&mut self, assert: UntypedAssert) -> UntypedAssert {
+        assert
+    }
 }
 
 #[allow(dead_code)]
@@ -925,6 +949,7 @@ pub trait UntypedConstantFolder {
                 tag: (),
                 type_: (),
                 field_map: _,
+                record_constructor: _,
             } => self.fold_constant_record(location, module, name, args),
 
             Constant::BitArray { location, segments } => {
@@ -1008,6 +1033,7 @@ pub trait UntypedConstantFolder {
             tag: (),
             type_: (),
             field_map: None,
+            record_constructor: None,
         }
     }
 
@@ -1088,6 +1114,7 @@ pub trait UntypedConstantFolder {
                 tag,
                 type_,
                 field_map,
+                record_constructor,
             } => {
                 let args = args
                     .into_iter()
@@ -1104,6 +1131,7 @@ pub trait UntypedConstantFolder {
                     tag,
                     type_,
                     field_map,
+                    record_constructor,
                 }
             }
 
