@@ -17,10 +17,11 @@ pub use self::project_compiler::{Built, Options, ProjectCompiler};
 pub use self::telemetry::{NullTelemetry, Telemetry};
 
 use crate::ast::{
-    self, CallArg, CustomType, DefinitionLocation, TypeAst, TypedArg, TypedDefinition, TypedExpr,
-    TypedFunction, TypedPattern, TypedRecordConstructor, TypedStatement,
+    self, CallArg, CustomType, DefinitionLocation, TypeAst, TypedArg, TypedConstant,
+    TypedDefinition, TypedExpr, TypedFunction, TypedPattern, TypedRecordConstructor,
+    TypedStatement,
 };
-use crate::type_::Type;
+use crate::type_::{Type, TypedCallArg};
 use crate::{
     ast::{Definition, SrcSpan, TypedModule},
     config::{self, PackageConfig},
@@ -34,7 +35,7 @@ use camino::Utf8PathBuf;
 use ecow::EcoString;
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
-use std::fmt::Debug;
+use std::fmt::{Debug, Display};
 use std::sync::Arc;
 use std::time::SystemTime;
 use std::{collections::HashMap, ffi::OsString, fs::DirEntry, iter::Peekable, process};
@@ -187,9 +188,9 @@ pub enum Mode {
 }
 
 impl Mode {
-    /// Returns `true` if the mode includes test code.
+    /// Returns `true` if the mode includes development code.
     ///
-    pub fn includes_tests(&self) -> bool {
+    pub fn includes_dev_code(&self) -> bool {
         match self {
             Self::Dev | Self::Lsp => true,
             Self::Prod => false,
@@ -205,10 +206,10 @@ impl Mode {
 }
 
 #[test]
-fn mode_includes_tests() {
-    assert!(Mode::Dev.includes_tests());
-    assert!(Mode::Lsp.includes_tests());
-    assert!(!Mode::Prod.includes_tests());
+fn mode_includes_dev_code() {
+    assert!(Mode::Dev.includes_dev_code());
+    assert!(Mode::Lsp.includes_dev_code());
+    assert!(!Mode::Prod.includes_dev_code());
 }
 
 #[derive(Debug)]
@@ -256,10 +257,6 @@ impl Module {
         path
     }
 
-    pub fn is_test(&self) -> bool {
-        self.origin == Origin::Test
-    }
-
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
         self.ast.find_node(byte_index)
     }
@@ -337,6 +334,18 @@ pub struct UnqualifiedImport<'a> {
     pub location: &'a SrcSpan,
 }
 
+/// The position of a located expression. Used to determine extra context,
+/// such as whether to provide label completions if the expression is in
+/// argument position.
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum ExpressionPosition<'a> {
+    Expression,
+    ArgumentOrLabel {
+        called_function: &'a TypedExpr,
+        function_arguments: &'a [TypedCallArg],
+    },
+}
+
 #[derive(Debug, Clone, PartialEq)]
 pub enum Located<'a> {
     Pattern(&'a TypedPattern),
@@ -345,7 +354,10 @@ pub enum Located<'a> {
         pattern: &'a TypedPattern,
     },
     Statement(&'a TypedStatement),
-    Expression(&'a TypedExpr),
+    Expression {
+        expression: &'a TypedExpr,
+        position: ExpressionPosition<'a>,
+    },
     ModuleStatement(&'a TypedDefinition),
     VariantConstructorDefinition(&'a TypedRecordConstructor),
     FunctionBody(&'a TypedFunction),
@@ -361,6 +373,7 @@ pub enum Located<'a> {
         name: &'a EcoString,
         layer: ast::Layer,
     },
+    Constant(&'a TypedConstant),
 }
 
 impl<'a> Located<'a> {
@@ -385,7 +398,7 @@ impl<'a> Located<'a> {
             Self::Pattern(pattern) => pattern.definition_location(),
             Self::Statement(statement) => statement.definition_location(),
             Self::FunctionBody(statement) => None,
-            Self::Expression(expression) => expression.definition_location(),
+            Self::Expression { expression, .. } => expression.definition_location(),
             Self::ModuleStatement(Definition::Import(import)) => Some(DefinitionLocation {
                 module: Some(import.module.clone()),
                 span: SrcSpan { start: 0, end: 0 },
@@ -423,6 +436,7 @@ impl<'a> Located<'a> {
                 module: Some((*name).clone()),
                 span: SrcSpan::new(0, 0),
             }),
+            Self::Constant(constant) => constant.definition_location(),
         }
     }
 
@@ -430,9 +444,10 @@ impl<'a> Located<'a> {
         match self {
             Located::Pattern(pattern) => Some(pattern.type_()),
             Located::Statement(statement) => Some(statement.type_()),
-            Located::Expression(typed_expr) => Some(typed_expr.type_()),
+            Located::Expression { expression, .. } => Some(expression.type_()),
             Located::Arg(arg) => Some(arg.type_.clone()),
             Located::Label(_, type_) | Located::Annotation { type_, .. } => Some(type_.clone()),
+            Located::Constant(constant) => Some(constant.type_()),
 
             Located::PatternSpread { .. } => None,
             Located::ModuleStatement(definition) => None,
@@ -547,6 +562,7 @@ pub fn type_constructor_from_modules(
 pub enum Origin {
     Src,
     Test,
+    Dev,
 }
 
 impl Origin {
@@ -557,6 +573,32 @@ impl Origin {
     pub fn is_src(&self) -> bool {
         matches!(self, Self::Src)
     }
+
+    /// Returns `true` if the origin is [`Test`].
+    ///
+    /// [`Test`]: Origin::Test
+    #[must_use]
+    pub fn is_test(&self) -> bool {
+        matches!(self, Self::Test)
+    }
+
+    /// Returns `true` if the origin is [`Dev`].
+    ///
+    /// [`Dev`]: Origin::Dev
+    #[must_use]
+    pub fn is_dev(&self) -> bool {
+        matches!(self, Self::Dev)
+    }
+
+    /// Name of the folder containing the origin.
+    #[must_use]
+    pub fn folder_name(&self) -> &str {
+        match self {
+            Origin::Src => "src",
+            Origin::Test => "test",
+            Origin::Dev => "dev",
+        }
+    }
 }
 
 fn doc_comments_before<'a>(
