@@ -1,7 +1,10 @@
 use super::*;
 use crate::{
     ast::{Arg, Function, ModuleConstant, Publicity},
-    type_::{Deprecation, expression::Implementations},
+    type_::{
+        Deprecation,
+        expression::{Implementations, Purity},
+    },
 };
 use ecow::EcoString;
 
@@ -45,6 +48,7 @@ fn parse_and_order(
                 can_run_on_erlang: true,
                 can_run_on_javascript: true,
             },
+            purity: Purity::Impure,
         })
         .collect_vec();
     let constants = constants
@@ -687,3 +691,39 @@ fn bug_2275() {
         vec![vec!["two"], vec!["one"]]
     );
 }
+
+#[test]
+fn let_assert_message() {
+    let functions = [
+        ("a", [].as_slice(), r#"{ let assert True = False as b() }"#),
+        ("b", [].as_slice(), r#"a()"#),
+    ];
+    assert_eq!(
+        parse_and_order(functions.as_slice(), [].as_slice()).unwrap(),
+        vec![vec!["b", "a"]]
+    );
+}
+
+#[test]
+fn assert_subject() {
+    let functions = [
+        ("a", [].as_slice(), r#"{ assert b() }"#),
+        ("b", [].as_slice(), r#"a()"#),
+    ];
+    assert_eq!(
+        parse_and_order(functions.as_slice(), [].as_slice()).unwrap(),
+        vec![vec!["b", "a"]]
+    );
+}
+
+#[test]
+fn assert_message() {
+    let functions = [
+        ("a", [].as_slice(), r#"{ assert False as b() }"#),
+        ("b", [].as_slice(), r#"a()"#),
+    ];
+    assert_eq!(
+        parse_and_order(functions.as_slice(), [].as_slice()).unwrap(),
+        vec![vec!["b", "a"]]
+    );
+}
