@@ -7,7 +7,7 @@ mod pattern;
 mod tests;
 
 use crate::build::{Target, module_erlang_name};
-use crate::strings::convert_string_escape_chars;
+use crate::strings::{convert_string_escape_chars, to_snake_case};
 use crate::type_::is_prelude_module;
 use crate::{
     Result,
@@ -22,7 +22,6 @@ use crate::{
 };
 use camino::Utf8Path;
 use ecow::{EcoString, eco_format};
-use heck::ToSnakeCase;
 use im::HashSet;
 use itertools::Itertools;
 use pattern::pattern;
@@ -34,20 +33,14 @@ use vec1::Vec1;
 const INDENT: isize = 4;
 const MAX_COLUMNS: isize = 80;
 
-fn module_name_to_erlang(module: &str) -> Document<'_> {
-    EcoString::from(module.replace('/', "@")).to_doc()
-}
-
 fn module_name_atom(module: &str) -> Document<'static> {
-    atom_string(module.replace('/', "@"))
+    atom_string(module.replace('/', "@").into())
 }
 
 #[derive(Debug, Clone)]
 struct Env<'a> {
     module: &'a str,
     function: &'a str,
-    src_path: &'a Utf8Path,
-    project_root: &'a Utf8Path,
     line_numbers: &'a LineNumbers,
     needs_function_docs: bool,
     echo_used: bool,
@@ -56,21 +49,13 @@ struct Env<'a> {
 }
 
 impl<'env> Env<'env> {
-    pub fn new(
-        module: &'env str,
-        src_path: &'env Utf8Path,
-        project_root: &'env Utf8Path,
-        function: &'env str,
-        line_numbers: &'env LineNumbers,
-    ) -> Self {
+    pub fn new(module: &'env str, function: &'env str, line_numbers: &'env LineNumbers) -> Self {
         let vars: im::HashMap<_, _> = std::iter::once(("_".into(), 0)).collect();
         Self {
             current_scope_vars: vars.clone(),
             erl_function_scope_vars: vars,
             needs_function_docs: false,
             echo_used: false,
-            src_path,
-            project_root,
             line_numbers,
             function,
             module,
@@ -141,26 +126,19 @@ pub fn records(module: &TypedModule) -> Vec<(&str, String)> {
 }
 
 pub fn record_definition(name: &str, fields: &[(&str, Arc<Type>)]) -> String {
-    let name = &name.to_snake_case();
+    let name = to_snake_case(name);
     let type_printer = TypePrinter::new("").var_as_any();
     let fields = fields.iter().map(move |(name, type_)| {
         let type_ = type_printer.print(type_);
-        docvec![atom_string((*name).to_string()), " :: ", type_.group()]
+        docvec![atom_string((*name).into()), " :: ", type_.group()]
     });
     let fields = break_("", "")
         .append(join(fields, break_(",", ", ")))
         .nest(INDENT)
         .append(break_("", ""))
         .group();
-    docvec![
-        "-record(",
-        atom_string(name.to_string()),
-        ", {",
-        fields,
-        "}).",
-        line()
-    ]
-    .to_pretty_string(MAX_COLUMNS)
+    docvec!["-record(", atom_string(name), ", {", fields, "}).", line()]
+        .to_pretty_string(MAX_COLUMNS)
 }
 
 pub fn module<'a>(
@@ -236,7 +214,13 @@ fn module_document<'a>(
     };
 
     let src_path_full = &module.type_info.src_path;
-    let src_path_relative = src_path_full.strip_prefix(root).unwrap_or(src_path_full);
+    let src_path_relative = EcoString::from(
+        src_path_full
+            .strip_prefix(root)
+            .unwrap_or(src_path_full)
+            .as_str(),
+    )
+    .replace("\\", "\\\\");
 
     let mut needs_function_docs = false;
     let mut echo_used = false;
@@ -247,8 +231,7 @@ fn module_document<'a>(
             &module.name,
             module.type_info.is_internal,
             line_numbers,
-            src_path_relative,
-            root,
+            src_path_relative.clone(),
         ) {
             needs_function_docs = needs_function_docs || env.needs_function_docs;
             echo_used = echo_used || env.echo_used;
@@ -285,7 +268,11 @@ fn module_document<'a>(
     let module = docvec![
         header,
         "-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).",
-        lines(2),
+        line(),
+        "-define(FILEPATH, \"",
+        src_path_relative,
+        "\").",
+        line(),
         exports,
         documentation_directive,
         module_doc,
@@ -324,7 +311,7 @@ fn register_imports(
             if implementations.supports(Target::Erlang) {
                 let function_name = escape_erlang_existing_name(name);
                 exports.push(
-                    atom_string(function_name.to_string())
+                    atom_string(function_name.into())
                         .append("/")
                         .append(args.len()),
                 )
@@ -368,7 +355,7 @@ fn register_imports(
             };
             // Type Exports
             type_exports.push(
-                erl_safe_type_name(name.to_snake_case())
+                erl_safe_type_name(to_snake_case(name))
                     .to_doc()
                     .append("/")
                     .append(typed_parameters.len()),
@@ -382,7 +369,7 @@ fn register_imports(
                 let constructors = constructors
                     .iter()
                     .map(|c| {
-                        let name = atom_string(c.name.to_snake_case());
+                        let name = atom_string(to_snake_case(&c.name));
                         if c.arguments.is_empty() {
                             name
                         } else {
@@ -402,7 +389,7 @@ fn register_imports(
             );
             let doc = if *opaque { "-opaque " } else { "-type " }
                 .to_doc()
-                .append(erl_safe_type_name(name.to_snake_case()))
+                .append(erl_safe_type_name(to_snake_case(name)))
                 .append("(")
                 .append(params)
                 .append(") :: ")
@@ -424,8 +411,7 @@ fn module_statement<'a>(
     module: &'a str,
     is_internal_module: bool,
     line_numbers: &'a LineNumbers,
-    src_path: &'a Utf8Path,
-    project_root: &'a Utf8Path,
+    src_path: EcoString,
 ) -> Option<(Document<'a>, Env<'a>)> {
     match statement {
         Definition::TypeAlias(TypeAlias { .. })
@@ -433,14 +419,9 @@ fn module_statement<'a>(
         | Definition::Import(Import { .. })
         | Definition::ModuleConstant(ModuleConstant { .. }) => None,
 
-        Definition::Function(function) => module_function(
-            function,
-            module,
-            is_internal_module,
-            line_numbers,
-            src_path,
-            project_root,
-        ),
+        Definition::Function(function) => {
+            module_function(function, module, is_internal_module, line_numbers, src_path)
+        }
     }
 }
 
@@ -449,8 +430,7 @@ fn module_function<'a>(
     module: &'a str,
     is_internal_module: bool,
     line_numbers: &'a LineNumbers,
-    src_path: &'a Utf8Path,
-    project_root: &'a Utf8Path,
+    src_path: EcoString,
 ) -> Option<(Document<'a>, Env<'a>)> {
     // Private external functions don't need to render anything, the underlying
     // Erlang implementation is used directly at the call site.
@@ -471,7 +451,7 @@ fn module_function<'a>(
     let function_name = escape_erlang_existing_name(function_name);
     let file_attribute = file_attribute(src_path, function, line_numbers);
 
-    let mut env = Env::new(module, src_path, project_root, function_name, line_numbers);
+    let mut env = Env::new(module, function_name, line_numbers);
     let var_usages = collect_type_var_usages(
         HashMap::new(),
         std::iter::once(&function.return_type).chain(function.arguments.iter().map(|a| &a.type_)),
@@ -530,7 +510,7 @@ fn module_function<'a>(
             attributes,
             line(),
             spec,
-            atom_string(escape_erlang_existing_name(function_name).to_string()),
+            atom_string(escape_erlang_existing_name(function_name).into()),
             arguments,
             " ->",
             line().append(body).nest(INDENT).group(),
@@ -541,12 +521,11 @@ fn module_function<'a>(
 }
 
 fn file_attribute<'a>(
-    path: &'a Utf8Path,
+    path: EcoString,
     function: &'a TypedFunction,
     line_numbers: &'a LineNumbers,
 ) -> Document<'a> {
     let line = line_numbers.line_number(function.location.start);
-    let path = EcoString::from(path.as_str()).replace("\\", "\\\\");
     docvec!["-file(\"", path, "\", ", line, ")."]
 }
 
@@ -656,7 +635,7 @@ fn fun_spec<'a>(
         .group()
 }
 
-fn atom_string(value: String) -> Document<'static> {
+fn atom_string(value: EcoString) -> Document<'static> {
     escape_atom_string(value).to_doc()
 }
 
@@ -678,13 +657,12 @@ fn atom(value: &str) -> Document<'_> {
     }
 }
 
-pub fn escape_atom_string(value: String) -> EcoString {
+pub fn escape_atom_string(value: EcoString) -> EcoString {
     if is_erlang_reserved_word(&value) {
         // Escape because of keyword collision
         eco_format!("'{value}'")
     } else if atom_pattern().is_match(&value) {
-        // No need to escape
-        EcoString::from(value)
+        value
     } else {
         // Escape because of characters contained
         eco_format!("'{value}'")
@@ -884,11 +862,21 @@ fn const_segment<'a>(
     )
 }
 
-fn statement<'a>(statement: &'a TypedStatement, env: &mut Env<'a>) -> Document<'a> {
+enum Position {
+    Tail,
+    NotTail,
+}
+
+fn statement<'a>(
+    statement: &'a TypedStatement,
+    env: &mut Env<'a>,
+    position: Position,
+) -> Document<'a> {
     match statement {
         Statement::Expression(e) => expr(e, env),
-        Statement::Assignment(a) => assignment(a, env),
+        Statement::Assignment(a) => assignment(a, env, position),
         Statement::Use(use_) => expr(&use_.call, env),
+        Statement::Assert(a) => assert(a, env),
     }
 }
 
@@ -923,7 +911,7 @@ fn expr_segment<'a>(
         }
 
         _ => {
-            let inner_expr = expr(expression, env).surround("(", ")");
+            let inner_expr = maybe_block_expr(expression, env).surround("(", ")");
             // The value of size must be a non-negative integer, we use lists:max here to ensure
             // it is at least 0;
             let value_guard = ":(lists:max(["
@@ -1041,7 +1029,12 @@ fn statement_sequence<'a>(statements: &'a [TypedStatement], env: &mut Env<'a>) -
     let count = statements.len();
     let mut documents = Vec::with_capacity(count * 3);
     for (i, expression) in statements.iter().enumerate() {
-        documents.push(statement(expression, env).group());
+        let position = if i + 1 == count {
+            Position::Tail
+        } else {
+            Position::NotTail
+        };
+        documents.push(statement(expression, env, position).group());
 
         if i + 1 < count {
             // This isn't the final expression so add the delimeters
@@ -1158,37 +1151,126 @@ fn binop_documents<'a>(left: Document<'a>, op: &'static str, right: Document<'a>
 
 fn let_assert<'a>(
     value: &'a TypedExpr,
-    pat: &'a TypedPattern,
+    pattern: &'a TypedPattern,
     env: &mut Env<'a>,
     message: Option<&'a TypedExpr>,
+    position: Position,
+    location: SrcSpan,
 ) -> Document<'a> {
+    // If the pattern will never fail, like a tuple or a simple variable, we
+    // simply treat it as if it were a `let` assignment.
+    if pattern.always_matches() {
+        return let_(value, pattern, env);
+    }
+
     let mut vars: Vec<&str> = vec![];
-    let body = maybe_block_expr(value, env);
-    let (subject_var, subject_definition) = if value.is_var() {
-        (body, nil())
+    let subject = maybe_block_expr(value, env);
+
+    // The code we generated for a `let assert` assignment looks something like
+    // this. For this Gleam code:
+    //
+    // ```gleam
+    // let assert [a, b, c] = [1, 2, 3]
+    // ```
+    //
+    // We generate (roughly) the following Erlang:
+    //
+    // ```erlang
+    // {A, B, C} = case [1, 2, 3] of
+    //   [A, B, C] -> {A, B, C};
+    //   _ -> erlang:error(...)
+    // end.
+    // ```
+    // This is the most efficient way to properly extract all the required
+    // variables from the pattern. However, if the `let assert` assignment is
+    // the last in a block, like this:
+    //
+    // ```gleam
+    // let x = {
+    //   let assert [a, b, c] = [1, 2, 3]
+    // }
+    // ```
+    //
+    // The generated Erlang code will end up assigning the value `#(1, 2, 3)`
+    // to the variable `x`, instead of `[1, 2, 3]`. In this case, we must
+    // generate slightly different code. Since we know we won't be using the
+    // bound variables anywhere (there is nothing else in this scope to
+    // reference them), we can safely remove the assignment from the generated
+    // code, and generate the following:
+    //
+    // ```erlang
+    // X = begin
+    //   _assert_subject = [1, 2, 3]
+    //   case _assert_subject of
+    //     [A, B, C] -> _assert_subject;
+    //     _ -> erlang:error(...)
+    //   end
+    // end.
+    // ```
+    //
+    // That correctly assigns `[1, 2, 3]` to the `x` variable.
+    //
+    let is_tail = match position {
+        Position::Tail => true,
+        Position::NotTail => false,
+    };
+
+    let (subject_assignment, subject) = if is_tail && !value.is_var() {
+        let variable = env.next_local_var_name(ASSERT_SUBJECT_VARIABLE);
+        let assignment = docvec![variable.clone(), " = ", subject, ",", line()];
+        (assignment, variable)
     } else {
-        let var = env.next_local_var_name(ASSERT_SUBJECT_VARIABLE);
-        let definition = docvec![var.clone(), " = ", body, ",", line()];
-        (var, definition)
+        (nil(), subject)
     };
 
     let mut guards = vec![];
-    let check_pattern = pattern::to_doc_discarding_all(pat, &mut vars, env, &mut guards);
+    let pattern_document = pattern::to_doc(pattern, &mut vars, env, &mut guards);
     let clause_guard = optional_clause_guard(None, guards, env);
 
-    // We don't take the guards from the assign pattern or we would end up with
-    // all the same guards repeated twice!
-    let assign_pattern = pattern::to_doc(pat, &mut vars, env, &mut vec![]);
     let message = match message {
         Some(message) => expr(message, env),
         None => string("Pattern match failed, no pattern matched the value."),
     };
 
+    let value_document = match vars.as_slice() {
+        _ if is_tail => subject.clone(),
+        [] => "nil".to_doc(),
+        [variable] => env.local_var_name(variable),
+        variables => {
+            let variables = variables
+                .iter()
+                .map(|variable| env.local_var_name(variable));
+            docvec![
+                break_("{", "{"),
+                join(variables, break_(",", ", ")).nest(INDENT),
+                "}"
+            ]
+            .group()
+        }
+    };
+
+    let assignment = match vars.as_slice() {
+        _ if is_tail => nil(),
+        [] => nil(),
+        [variable] => env.next_local_var_name(variable).append(" = "),
+        variables => {
+            let variables = variables
+                .iter()
+                .map(|variable| env.next_local_var_name(variable));
+            docvec![
+                break_("{", "{"),
+                join(variables, break_(",", ", ")).nest(INDENT),
+                "} = "
+            ]
+            .group()
+        }
+    };
+
     let clauses = docvec![
-        check_pattern.clone(),
+        pattern_document,
         clause_guard,
         " -> ",
-        subject_var.clone(),
+        value_document,
         ";",
         line(),
         env.next_local_var_name(ASSERT_FAIL_VARIABLE),
@@ -1198,8 +1280,14 @@ fn let_assert<'a>(
             erlang_error(
                 "let_assert",
                 &message,
-                pat.location(),
-                vec![("value", env.local_var_name(ASSERT_FAIL_VARIABLE))],
+                location,
+                vec![
+                    ("value", env.local_var_name(ASSERT_FAIL_VARIABLE)),
+                    ("start", location.start.to_doc()),
+                    ("'end'", value.location().end.to_doc()),
+                    ("pattern_start", pattern.location().start.to_doc()),
+                    ("pattern_end", pattern.location().end.to_doc()),
+                ],
                 env,
             )
             .nest(INDENT)
@@ -1207,10 +1295,10 @@ fn let_assert<'a>(
         .nest(INDENT)
     ];
     docvec![
-        subject_definition,
-        assign_pattern,
-        " = case ",
-        subject_var,
+        subject_assignment,
+        assignment,
+        "case ",
+        subject,
         " of",
         docvec![line(), clauses].nest(INDENT),
         line(),
@@ -1279,12 +1367,12 @@ fn var<'a>(name: &'a str, constructor: &'a ValueConstructor, env: &mut Env<'a>)
                     .to_doc()
                     .append(chars.clone())
                     .append(") -> {")
-                    .append(atom_string(record_name.to_snake_case()))
+                    .append(atom_string(to_snake_case(record_name)))
                     .append(", ")
                     .append(chars)
                     .append("} end")
             }
-            _ => atom_string(record_name.to_snake_case()),
+            _ => atom_string(to_snake_case(record_name)),
         },
 
         ValueConstructorVariant::LocalVariable { .. } => env.local_var_name(name),
@@ -1367,12 +1455,12 @@ fn const_inline<'a>(literal: &'a TypedConstant, env: &mut Env<'a>) -> Document<'
             tag, type_, args, ..
         } if args.is_empty() => match type_.deref() {
             Type::Fn { args, .. } => record_constructor_function(tag, args.len()),
-            _ => atom_string(tag.to_snake_case()),
+            _ => atom_string(to_snake_case(tag)),
         },
 
         Constant::Record { tag, args, .. } => {
             let args = args.iter().map(|a| const_inline(&a.value, env));
-            let tag = atom_string(tag.to_snake_case());
+            let tag = atom_string(to_snake_case(tag));
             tuple(std::iter::once(tag).chain(args))
         }
 
@@ -1400,7 +1488,7 @@ fn record_constructor_function(tag: &EcoString, arity: usize) -> Document<'_> {
         .to_doc()
         .append(chars.clone())
         .append(") -> {")
-        .append(atom_string(tag.to_snake_case()))
+        .append(atom_string(to_snake_case(tag)))
         .append(", ")
         .append(chars)
         .append("} end")
@@ -1700,7 +1788,7 @@ fn module_fn_with_args<'a>(
     if module == env.module {
         atom(name).append(args)
     } else {
-        atom_string(module.replace('/', "@"))
+        atom_string(module.replace('/', "@").into())
             .append(":")
             .append(atom(name))
             .append(args)
@@ -1724,7 +1812,7 @@ fn docs_args_call<'a>(
                     ..
                 },
             ..
-        } => tuple(std::iter::once(atom_string(name.to_snake_case())).chain(args)),
+        } => tuple(std::iter::once(atom_string(to_snake_case(name))).chain(args)),
 
         TypedExpr::Var {
             constructor:
@@ -1790,9 +1878,9 @@ fn docs_args_call<'a>(
             // This also enables an optimisation in the Erlang compiler in which
             // some Erlang BIFs can be replaced with literals if their arguments
             // are literals, such as `binary_to_atom`.
-            atom_string(module_erlang_name(module).to_string())
+            atom_string(module_erlang_name(module))
                 .append(":")
-                .append(atom_string(name.to_string()))
+                .append(atom_string(name.into()))
                 .append(args)
         }
 
@@ -1844,7 +1932,7 @@ fn record_update<'a>(
     let vars = env.current_scope_vars.clone();
 
     let document = docvec![
-        assignment(record, env),
+        assignment(record, env, Position::NotTail),
         ",",
         line(),
         call(constructor, args, env)
@@ -1915,22 +2003,8 @@ fn panic<'a>(location: SrcSpan, message: Option<&'a TypedExpr>, env: &mut Env<'a
 
 fn echo<'a>(body: Document<'a>, location: &SrcSpan, env: &mut Env<'a>) -> Document<'a> {
     env.echo_used = true;
-
-    let relative_path = env
-        .src_path
-        .strip_prefix(env.project_root)
-        .unwrap_or(env.src_path)
-        .as_str();
-
-    let relative_path_doc = EcoString::from(relative_path)
-        .replace("\\", "\\\\")
-        .to_doc();
-
-    let relative_path_doc = docvec!["\"", relative_path_doc, "\""];
-
     "echo".to_doc().append(wrap_args(vec![
         body,
-        relative_path_doc,
         env.line_numbers.line_number(location.start).to_doc(),
     ]))
 }
@@ -1948,9 +2022,22 @@ fn erlang_error<'a>(
         ",",
         line(),
         "message => ",
-        message.clone()
+        message.clone(),
+        ",",
+        line(),
+        "file => <<?FILEPATH/utf8>>,",
+        line(),
+        "module => ",
+        env.module.to_doc().surround("<<\"", "\"/utf8>>"),
+        ",",
+        line(),
+        "function => ",
+        string(env.function),
+        ",",
+        line(),
+        "line => ",
+        env.line_numbers.line_number(location.start),
     ];
-
     for (key, value) in fields {
         fields_doc = fields_doc
             .append(",")
@@ -1959,23 +2046,7 @@ fn erlang_error<'a>(
             .append(" => ")
             .append(value);
     }
-    let fields_doc = fields_doc
-        .append(",")
-        .append(line())
-        .append("module => ")
-        .append(env.module.to_doc().surround("<<\"", "\"/utf8>>"))
-        .append(",")
-        .append(line())
-        .append("function => ")
-        .append(string(env.function))
-        .append(",")
-        .append(line())
-        .append("line => ")
-        .append(env.line_numbers.line_number(location.start));
-    let error = "#{"
-        .to_doc()
-        .append(fields_doc.group().nest(INDENT))
-        .append("}");
+    let error = docvec!["#{", fields_doc.group().nest(INDENT), "}"];
     docvec!["erlang:error", wrap_args([error.group()])]
 }
 
@@ -2034,7 +2105,7 @@ fn expr<'a>(expression: &'a TypedExpr, env: &mut Env<'a>) -> Document<'a> {
         TypedExpr::ModuleSelect {
             constructor: ModuleValueConstructor::Record { name, arity: 0, .. },
             ..
-        } => atom_string(name.to_snake_case()),
+        } => atom_string(to_snake_case(name)),
 
         TypedExpr::ModuleSelect {
             constructor: ModuleValueConstructor::Constant { literal, .. },
@@ -2143,20 +2214,419 @@ fn pipeline<'a>(
     documents.to_doc()
 }
 
-fn assignment<'a>(assignment: &'a TypedAssignment, env: &mut Env<'a>) -> Document<'a> {
+fn assignment<'a>(
+    assignment: &'a TypedAssignment,
+    env: &mut Env<'a>,
+    position: Position,
+) -> Document<'a> {
     match &assignment.kind {
         AssignmentKind::Let | AssignmentKind::Generated => {
             let_(&assignment.value, &assignment.pattern, env)
         }
-        AssignmentKind::Assert { message, .. } => let_assert(
+        AssignmentKind::Assert {
+            message, location, ..
+        } => let_assert(
             &assignment.value,
             &assignment.pattern,
             env,
-            message.as_deref(),
+            message.as_ref(),
+            position,
+            *location,
         ),
     }
 }
 
+fn assert<'a>(assert: &'a TypedAssert, env: &mut Env<'a>) -> Document<'a> {
+    let Assert {
+        value,
+        location,
+        message,
+    } = assert;
+
+    let message = match message {
+        Some(message) => expr(message, env),
+        None => string("Assertion failed."),
+    };
+
+    let mut assignments = Vec::new();
+
+    let (subject, mut fields) = match value {
+        TypedExpr::Call { fun, args, .. } => assert_call(fun, args, &mut assignments, env),
+        TypedExpr::BinOp {
+            name, left, right, ..
+        } => {
+            let operator = match name {
+                BinOp::And => {
+                    return assert_and(left, right, message, *location, env);
+                }
+                BinOp::Or => {
+                    return assert_or(left, right, message, *location, env);
+                }
+                BinOp::Eq => "=:=",
+                BinOp::NotEq => "/=",
+                BinOp::LtInt | BinOp::LtFloat => "<",
+                BinOp::LtEqInt | BinOp::LtEqFloat => "=<",
+                BinOp::GtInt | BinOp::GtFloat => ">",
+                BinOp::GtEqInt | BinOp::GtEqFloat => ">=",
+                BinOp::AddInt
+                | BinOp::AddFloat
+                | BinOp::SubInt
+                | BinOp::SubFloat
+                | BinOp::MultInt
+                | BinOp::MultFloat
+                | BinOp::DivInt
+                | BinOp::DivFloat
+                | BinOp::RemainderInt
+                | BinOp::Concatenate => {
+                    panic!("Non-boolean operators cannot appear here in well-typed code")
+                }
+            };
+
+            let left_document = assign_to_variable(left, &mut assignments, env);
+            let right_document = assign_to_variable(right, &mut assignments, env);
+            (
+                binop_documents(left_document.clone(), operator, right_document.clone()),
+                vec![
+                    ("kind", atom("binary_operator")),
+                    ("operator", atom(name.name())),
+                    (
+                        "left",
+                        asserted_expression(
+                            AssertExpression::from_expression(left),
+                            Some(left_document),
+                            left.location(),
+                        ),
+                    ),
+                    (
+                        "right",
+                        asserted_expression(
+                            AssertExpression::from_expression(right),
+                            Some(right_document),
+                            right.location(),
+                        ),
+                    ),
+                ],
+            )
+        }
+
+        _ => (
+            maybe_block_expr(value, env),
+            vec![
+                ("kind", atom("expression")),
+                (
+                    "expression",
+                    asserted_expression(
+                        AssertExpression::from_expression(value),
+                        Some("false".to_doc()),
+                        value.location(),
+                    ),
+                ),
+            ],
+        ),
+    };
+
+    fields.push(("start", location.start.to_doc()));
+    fields.push(("'end'", value.location().end.to_doc()));
+    fields.push(("expression_start", value.location().start.to_doc()));
+
+    let clauses = docvec![
+        line(),
+        "true -> nil;",
+        line(),
+        "false -> ",
+        erlang_error("assert", &message, *location, fields, env),
+    ];
+
+    docvec![
+        assignments,
+        "case ",
+        subject,
+        " of",
+        clauses.nest(INDENT),
+        line(),
+        "end"
+    ]
+}
+
+fn assert_call<'a>(
+    function: &'a TypedExpr,
+    arguments: &'a Vec<CallArg<TypedExpr>>,
+    assignments: &mut Vec<Document<'a>>,
+    env: &mut Env<'a>,
+) -> (Document<'a>, Vec<(&'static str, Document<'a>)>) {
+    let argument_variables = arguments
+        .iter()
+        .map(|argument| assign_to_variable(&argument.value, assignments, env))
+        .collect_vec();
+
+    let arguments = join(
+        argument_variables
+            .iter()
+            .zip(arguments)
+            .map(|(variable, argument)| {
+                asserted_expression(
+                    AssertExpression::from_expression(&argument.value),
+                    Some(variable.clone()),
+                    argument.location(),
+                )
+            }),
+        break_(",", ", "),
+    )
+    .nest(INDENT)
+    .surround("[", "]");
+
+    (
+        docs_args_call(function, argument_variables, env),
+        vec![("kind", atom("function_call")), ("arguments", arguments)],
+    )
+}
+
+/// In Gleam, the `&&` operator is short-circuiting, meaning that we can't
+/// pre-evaluate both sides of it, and use them in the exception that is
+/// thrown.
+/// Instead, we need to implement this short-circuiting logic ourself.
+///
+/// If we short-circuit, we must leave the second expression unevaluated,
+/// and signal that using the `unevaluated` variant, as detailed in the
+/// exception format. For the first expression, we know it must be `false`,
+/// otherwise we would have continued by evaluating the second expression.
+///
+/// Similarly, if we do evaluate the second expression and fail, we know
+/// that the first expression must have evaluated to `true`, and the second
+/// to `false`. This way, we avoid needing to evaluate either expression
+/// twice.
+///
+/// The generated code then looks something like this:
+/// ```erlang
+/// case expr1 of
+///   true -> case expr2 of
+///     true -> true;
+///     false -> <throw exception>
+///   end;
+///   false -> <throw exception>
+/// end
+/// ```
+///
+fn assert_and<'a>(
+    left: &'a TypedExpr,
+    right: &'a TypedExpr,
+    message: Document<'a>,
+    location: SrcSpan,
+    env: &mut Env<'a>,
+) -> Document<'a> {
+    let left_kind = AssertExpression::from_expression(left);
+    let right_kind = AssertExpression::from_expression(right);
+
+    let fields_if_short_circuiting = vec![
+        ("kind", atom("binary_operator")),
+        ("operator", atom("&&")),
+        (
+            "left",
+            asserted_expression(left_kind, Some("false".to_doc()), left.location()),
+        ),
+        (
+            "right",
+            asserted_expression(AssertExpression::Unevaluated, None, right.location()),
+        ),
+        ("start", location.start.to_doc()),
+        ("'end'", right.location().end.to_doc()),
+        ("expression_start", left.location().start.to_doc()),
+    ];
+
+    let fields = vec![
+        ("kind", atom("binary_operator")),
+        ("operator", atom("&&")),
+        (
+            "left",
+            asserted_expression(left_kind, Some("true".to_doc()), left.location()),
+        ),
+        (
+            "right",
+            asserted_expression(right_kind, Some("false".to_doc()), right.location()),
+        ),
+        ("start", location.start.to_doc()),
+        ("'end'", right.location().end.to_doc()),
+        ("expression_start", left.location().start.to_doc()),
+    ];
+
+    let right_clauses = docvec![
+        line(),
+        "true -> nil;",
+        line(),
+        "false -> ",
+        erlang_error("assert", &message, location, fields, env),
+    ];
+
+    let left_clauses = docvec![
+        line(),
+        "true -> ",
+        docvec![
+            "case ",
+            maybe_block_expr(right, env),
+            " of",
+            right_clauses.nest(INDENT),
+            line(),
+            "end"
+        ]
+        .nest(INDENT),
+        ";",
+        line(),
+        "false -> ",
+        erlang_error(
+            "assert",
+            &message,
+            location,
+            fields_if_short_circuiting,
+            env
+        ),
+    ];
+
+    docvec![
+        "case ",
+        maybe_block_expr(left, env),
+        " of",
+        left_clauses.nest(INDENT),
+        line(),
+        "end"
+    ]
+}
+
+/// Similar to `&&`, `||` is also short-circuiting in Gleam. However, if `||`
+/// short-circuits, that's because the first expression evaluated to `true`,
+/// meaning the whole assertion succeeds. This allows us to directly use Erlang's
+/// `orelse` operator as the subject of the `case` expression.
+///
+/// The only difference is that due to the nature of `||`, if the assertion fails,
+/// we know that both sides must have evaluated to `false`, so we don't
+/// need to store the values of them in variables beforehand.
+fn assert_or<'a>(
+    left: &'a TypedExpr,
+    right: &'a TypedExpr,
+    message: Document<'a>,
+    location: SrcSpan,
+    env: &mut Env<'a>,
+) -> Document<'a> {
+    let fields = vec![
+        ("kind", atom("binary_operator")),
+        ("operator", atom("||")),
+        (
+            "left",
+            asserted_expression(
+                AssertExpression::from_expression(left),
+                Some("false".to_doc()),
+                left.location(),
+            ),
+        ),
+        (
+            "right",
+            asserted_expression(
+                AssertExpression::from_expression(right),
+                Some("false".to_doc()),
+                right.location(),
+            ),
+        ),
+        ("start", location.start.to_doc()),
+        ("'end'", right.location().end.to_doc()),
+        ("expression_start", left.location().start.to_doc()),
+    ];
+
+    let clauses = docvec![
+        line(),
+        "true -> nil;",
+        line(),
+        "false -> ",
+        erlang_error("assert", &message, location, fields, env),
+    ];
+
+    docvec![
+        "case ",
+        docvec![
+            maybe_block_expr(left, env),
+            " orelse ",
+            maybe_block_expr(right, env)
+        ]
+        .nest(INDENT),
+        " of",
+        clauses.nest(INDENT),
+        line(),
+        "end"
+    ]
+}
+
+fn assign_to_variable<'a>(
+    value: &'a TypedExpr,
+    assignments: &mut Vec<Document<'a>>,
+    env: &mut Env<'a>,
+) -> Document<'a> {
+    if value.is_var() {
+        expr(value, env)
+    } else {
+        let value = maybe_block_expr(value, env);
+        let variable = env.next_local_var_name(ASSERT_SUBJECT_VARIABLE);
+        let definition = docvec![variable.clone(), " = ", value, ",", line()];
+        assignments.push(definition);
+        variable
+    }
+}
+
+#[derive(Debug, Clone, Copy)]
+enum AssertExpression {
+    Literal,
+    Expression,
+    Unevaluated,
+}
+
+impl AssertExpression {
+    fn from_expression(expression: &TypedExpr) -> Self {
+        if expression.is_literal() {
+            Self::Literal
+        } else {
+            Self::Expression
+        }
+    }
+}
+
+fn asserted_expression(
+    kind: AssertExpression,
+    value: Option<Document<'_>>,
+    location: SrcSpan,
+) -> Document<'_> {
+    let kind = match kind {
+        AssertExpression::Literal => atom("literal"),
+        AssertExpression::Expression => atom("expression"),
+        AssertExpression::Unevaluated => atom("unevaluated"),
+    };
+
+    let start = location.start.to_doc();
+    let end = location.end.to_doc();
+
+    let value_field = if let Some(value) = value {
+        docvec!["value => ", value, ",", line()]
+    } else {
+        nil()
+    };
+
+    let fields_doc = docvec![
+        "kind => ",
+        kind,
+        ",",
+        line(),
+        value_field,
+        "start => ",
+        start,
+        ",",
+        line(),
+        // `end` is a keyword in Erlang, so we have to quote it
+        "'end' => ",
+        end,
+        line(),
+    ];
+
+    "#{".to_doc()
+        .append(fields_doc.group().nest(INDENT))
+        .append("}")
+}
+
 fn negate_with<'a>(op: &'static str, value: &'a TypedExpr, env: &mut Env<'a>) -> Document<'a> {
     docvec![op, maybe_block_expr(value, env)]
 }
@@ -2171,15 +2641,9 @@ fn tuple_index<'a>(tuple: &'a TypedExpr, index: u64, env: &mut Env<'a>) -> Docum
 
 fn module_select_fn<'a>(type_: Arc<Type>, module_name: &'a str, label: &'a str) -> Document<'a> {
     match crate::type_::collapse_links(type_).as_ref() {
-        Type::Fn { args, .. } => "fun "
-            .to_doc()
-            .append(module_name_to_erlang(module_name))
-            .append(":")
-            .append(atom(label))
-            .append("/")
-            .append(args.len()),
+        Type::Fn { args, .. } => function_reference(Some(module_name), label, args.len()),
 
-        _ => module_name_to_erlang(module_name)
+        _ => module_name_atom(module_name)
             .append(":")
             .append(atom(label))
             .append("()"),
@@ -2435,7 +2899,7 @@ fn type_var_ids(type_: &Type, ids: &mut HashMap<u64, u64>) {
     }
 }
 
-fn erl_safe_type_name(mut name: String) -> EcoString {
+fn erl_safe_type_name(mut name: EcoString) -> EcoString {
     if matches!(
         name.as_str(),
         "any"
@@ -2478,7 +2942,7 @@ fn erl_safe_type_name(mut name: String) -> EcoString {
             | "tuple"
     ) {
         name.push('_');
-        EcoString::from(name)
+        name
     } else {
         escape_atom_string(name)
     }
@@ -2568,7 +3032,7 @@ impl<'a> TypePrinter<'a> {
 
     fn print_type_app(&self, module: &str, name: &str, args: &[Arc<Type>]) -> Document<'static> {
         let args = join(args.iter().map(|a| self.print(a)), ", ".to_doc());
-        let name = erl_safe_type_name(name.to_snake_case()).to_doc();
+        let name = erl_safe_type_name(to_snake_case(name)).to_doc();
         if self.current_module == module {
             docvec![name, "(", args, ")"]
         } else {
