@@ -18,30 +18,22 @@ pub(super) fn pattern<'a>(
 fn print<'a>(
     p: &'a TypedPattern,
     vars: &mut Vec<&'a str>,
-    define_variables: bool,
     env: &mut Env<'a>,
     guards: &mut Vec<Document<'a>>,
 ) -> Document<'a> {
     match p {
         Pattern::Assign {
             name, pattern: p, ..
-        } if define_variables => {
+        } => {
             vars.push(name);
-            print(p, vars, define_variables, env, guards)
+            print(p, vars, env, guards)
                 .append(" = ")
                 .append(env.next_local_var_name(name))
         }
 
-        Pattern::Assign { pattern: p, .. } => print(p, vars, define_variables, env, guards),
-
-        Pattern::List { elements, tail, .. } => pattern_list(
-            elements,
-            tail.as_deref(),
-            vars,
-            define_variables,
-            env,
-            guards,
-        ),
+        Pattern::List { elements, tail, .. } => {
+            pattern_list(elements, tail.as_deref(), vars, env, guards)
+        }
 
         Pattern::Discard { .. } => "_".to_doc(),
 
@@ -60,13 +52,11 @@ fn print<'a>(
             }
         }
 
-        Pattern::Variable { name, .. } if define_variables => {
+        Pattern::Variable { name, .. } => {
             vars.push(name);
             env.next_local_var_name(name)
         }
 
-        Pattern::Variable { .. } => "_".to_doc(),
-
         Pattern::Int { value, .. } => int(value),
 
         Pattern::Float { value, .. } => float(value),
@@ -77,7 +67,7 @@ fn print<'a>(
             arguments: args,
             constructor: Inferred::Known(PatternConstructor { name, .. }),
             ..
-        } => tag_tuple_pattern(name, args, vars, define_variables, env, guards),
+        } => tag_tuple_pattern(name, args, vars, env, guards),
 
         Pattern::Constructor {
             constructor: Inferred::Unknown,
@@ -86,18 +76,16 @@ fn print<'a>(
             panic!("Erlang generation performed with uninferred pattern constructor")
         }
 
-        Pattern::Tuple { elements, .. } => tuple(
-            elements
-                .iter()
-                .map(|p| print(p, vars, define_variables, env, guards)),
-        ),
-
-        Pattern::BitArray { segments, .. } => {
-            bit_array(segments.iter().map(|s| {
-                pattern_segment(&s.value, &s.options, vars, define_variables, env, guards)
-            }))
+        Pattern::Tuple { elements, .. } => {
+            tuple(elements.iter().map(|p| print(p, vars, env, guards)))
         }
 
+        Pattern::BitArray { segments, .. } => bit_array(
+            segments
+                .iter()
+                .map(|s| pattern_segment(&s.value, &s.options, vars, env, guards)),
+        ),
+
         Pattern::StringPrefix {
             left_side_string,
             right_side_assignment,
@@ -105,8 +93,11 @@ fn print<'a>(
             ..
         } => {
             let right = match right_side_assignment {
-                AssignName::Variable(right) if define_variables => env.next_local_var_name(right),
-                AssignName::Variable(_) | AssignName::Discard(_) => "_".to_doc(),
+                AssignName::Variable(right) => {
+                    vars.push(right);
+                    env.next_local_var_name(right)
+                }
+                AssignName::Discard(_) => "_".to_doc(),
             };
 
             match left_side_assignment {
@@ -122,6 +113,7 @@ fn print<'a>(
                     //   to variables within the pattern, we first match the expected prefix length in
                     //   bytes, then use a guard clause to verify the content.
                     //
+                    vars.push(left_name);
                     let name = env.next_local_var_name(left_name);
                     guards.push(docvec![name.clone(), " =:= ", string(left_side_string)]);
                     docvec![
@@ -156,34 +148,23 @@ pub(super) fn to_doc<'a>(
     env: &mut Env<'a>,
     guards: &mut Vec<Document<'a>>,
 ) -> Document<'a> {
-    print(p, vars, true, env, guards)
-}
-
-pub(super) fn to_doc_discarding_all<'a>(
-    p: &'a TypedPattern,
-    vars: &mut Vec<&'a str>,
-    env: &mut Env<'a>,
-    guards: &mut Vec<Document<'a>>,
-) -> Document<'a> {
-    print(p, vars, false, env, guards)
+    print(p, vars, env, guards)
 }
 
 fn tag_tuple_pattern<'a>(
     name: &'a str,
     args: &'a [CallArg<TypedPattern>],
     vars: &mut Vec<&'a str>,
-    define_variables: bool,
     env: &mut Env<'a>,
     guards: &mut Vec<Document<'a>>,
 ) -> Document<'a> {
     if args.is_empty() {
-        atom_string(name.to_snake_case())
+        atom_string(to_snake_case(name))
     } else {
         tuple(
-            [atom_string(name.to_snake_case())].into_iter().chain(
-                args.iter()
-                    .map(|p| print(&p.value, vars, define_variables, env, guards)),
-            ),
+            [atom_string(to_snake_case(name))]
+                .into_iter()
+                .chain(args.iter().map(|p| print(&p.value, vars, env, guards))),
         )
     }
 }
@@ -192,7 +173,6 @@ fn pattern_segment<'a>(
     value: &'a TypedPattern,
     options: &'a [BitArrayOption<TypedPattern>],
     vars: &mut Vec<&'a str>,
-    define_variables: bool,
     env: &mut Env<'a>,
     guards: &mut Vec<Document<'a>>,
 ) -> Document<'a> {
@@ -207,13 +187,52 @@ fn pattern_segment<'a>(
         Pattern::Discard { .. }
         | Pattern::Variable { .. }
         | Pattern::Int { .. }
-        | Pattern::Float { .. } => print(
-            value,
-            &mut vars.borrow_mut(),
-            define_variables,
-            env,
-            &mut guards.borrow_mut(),
-        ),
+        | Pattern::Float { .. } => {
+            print(value, &mut vars.borrow_mut(), env, &mut guards.borrow_mut())
+        }
+
+        Pattern::Assign { name, pattern, .. } => {
+            vars.borrow_mut().push(name);
+            let variable_name = env.next_local_var_name(name);
+
+            match pattern.as_ref() {
+                // In Erlang, assignment patterns inside bit arrays are not allowed. So instead of
+                // generating `<<1 = A>>`, we  use guards, and generate `<<A>> when A =:= 1`.
+                Pattern::Int { value, .. } => {
+                    guards
+                        .borrow_mut()
+                        .push(docvec![variable_name.clone(), " =:= ", int(value)]);
+                    variable_name
+                }
+                Pattern::Float { value, .. } => {
+                    guards
+                        .borrow_mut()
+                        .push(docvec![variable_name.clone(), " =:= ", float(value)]);
+                    variable_name
+                }
+
+                // Here we do the same as for floats and ints, but we must calculate the size of
+                // the string first, so we can correctly match the bit array segment then compare
+                // it afterwards.
+                Pattern::String { value, .. } => {
+                    guards.borrow_mut().push(docvec![
+                        variable_name.clone(),
+                        " =:= ",
+                        string(value)
+                    ]);
+                    docvec![variable_name, ":", string_length_utf8_bytes(value)]
+                }
+
+                // Doing a pattern such as `<<_ as a>>` is the same as just `<<a>>`, so we treat it
+                // as such.
+                Pattern::Discard { .. } => variable_name,
+
+                // Any other pattern is invalid as a bit array segment. We already handle the case
+                // of `<<a as b>>` in the type-checker, and assignment patterns cannot be nested.
+                _ => panic!("Pattern segment match not recognised"),
+            }
+        }
+
         _ => panic!("Pattern segment match not recognised"),
     };
 
@@ -221,7 +240,6 @@ fn pattern_segment<'a>(
         Some(":".to_doc().append(print(
             value,
             &mut vars.borrow_mut(),
-            define_variables,
             env,
             &mut guards.borrow_mut(),
         )))
@@ -244,16 +262,15 @@ fn pattern_list<'a>(
     elements: &'a [TypedPattern],
     tail: Option<&'a TypedPattern>,
     vars: &mut Vec<&'a str>,
-    define_variables: bool,
     env: &mut Env<'a>,
     guards: &mut Vec<Document<'a>>,
 ) -> Document<'a> {
     let elements = join(
         elements
             .iter()
-            .map(|element| print(element, vars, define_variables, env, guards)),
+            .map(|element| print(element, vars, env, guards)),
         break_(",", ", "),
     );
-    let tail = tail.map(|tail| print(tail, vars, define_variables, env, guards));
+    let tail = tail.map(|tail| print(tail, vars, env, guards));
     list(elements, tail)
 }
