@@ -129,17 +129,6 @@ fn bit_array_declare_and_use_var() {
     );
 }
 
-#[test]
-fn negative_size() {
-    assert_erl!(
-        r#"
-pub fn main() {
-  <<1:size(-1)>>
-}
-"#,
-    );
-}
-
 // https://github.com/gleam-lang/gleam/issues/3050
 #[test]
 fn unicode_bit_array_1() {
@@ -204,3 +193,19 @@ pub fn main() {
 }"#
     );
 }
+
+// https://github.com/gleam-lang/gleam/issues/3944
+#[test]
+fn pipe_size_segment() {
+    assert_erl!(
+        "
+pub fn main() {
+  <<0xAE:size(5 |> identity)>>
+}
+
+fn identity(x) {
+  x
+}
+"
+    );
+}
