@@ -1,6 +1,6 @@
+mod decision;
 mod expression;
 mod import;
-mod pattern;
 #[cfg(test)]
 mod tests;
 mod typescript;
@@ -43,13 +43,15 @@ pub enum JavaScriptCodegenTarget {
 pub struct Generator<'a> {
     line_numbers: &'a LineNumbers,
     module: &'a TypedModule,
-    project_root: &'a Utf8Path,
     tracker: UsageTracker,
     module_scope: im::HashMap<EcoString, usize>,
     current_module_name_segments_count: usize,
     target_support: TargetSupport,
     typescript: TypeScriptDeclarations,
     stdlib_package: StdlibPackage,
+    /// Relative path to the module, surrounded in `"`s to make it a string, and with `\`s escaped
+    /// to `\\`.
+    src_path: EcoString,
 }
 
 impl<'a> Generator<'a> {
@@ -66,11 +68,18 @@ impl<'a> Generator<'a> {
         } = config;
         let current_module_name_segments_count = module.name.split('/').count();
 
+        let src_path = &module.type_info.src_path;
+        let src_path = src_path
+            .strip_prefix(project_root)
+            .unwrap_or(src_path)
+            .as_str();
+        let src_path = eco_format!("\"{src_path}\"").replace("\\", "\\\\");
+
         Self {
             current_module_name_segments_count,
             line_numbers,
-            project_root,
             module,
+            src_path,
             tracker: UsageTracker::default(),
             module_scope: Default::default(),
             target_support,
@@ -81,7 +90,7 @@ impl<'a> Generator<'a> {
 
     fn type_reference(&self) -> Document<'a> {
         if self.typescript == TypeScriptDeclarations::None {
-            return "".to_doc();
+            return nil();
         }
 
         // Get the name of the module relative the directory (similar to basename)
@@ -97,8 +106,6 @@ impl<'a> Generator<'a> {
     }
 
     pub fn compile(&mut self) -> Output<'a> {
-        let type_reference = self.type_reference();
-
         // Determine what JavaScript imports we need to generate
         let mut imports = self.collect_imports();
 
@@ -133,6 +140,14 @@ impl<'a> Generator<'a> {
             self.register_prelude_usage(&mut imports, "toList", None);
         };
 
+        if self.tracker.list_empty_class_used {
+            self.register_prelude_usage(&mut imports, "Empty", Some("$Empty"));
+        };
+
+        if self.tracker.list_non_empty_class_used {
+            self.register_prelude_usage(&mut imports, "NonEmpty", Some("$NonEmpty"));
+        };
+
         if self.tracker.prepend_used {
             self.register_prelude_usage(&mut imports, "prepend", Some("listPrepend"));
         };
@@ -179,66 +194,96 @@ impl<'a> Generator<'a> {
 
         if self.tracker.sized_integer_segment_used {
             self.register_prelude_usage(&mut imports, "sizedInt", None);
-        };
+        }
 
         if self.tracker.string_bit_array_segment_used {
             self.register_prelude_usage(&mut imports, "stringBits", None);
-        };
+        }
+
+        if self.tracker.string_utf16_bit_array_segment_used {
+            self.register_prelude_usage(&mut imports, "stringToUtf16", None);
+        }
+
+        if self.tracker.string_utf32_bit_array_segment_used {
+            self.register_prelude_usage(&mut imports, "stringToUtf32", None);
+        }
 
         if self.tracker.codepoint_bit_array_segment_used {
             self.register_prelude_usage(&mut imports, "codepointBits", None);
-        };
+        }
+
+        if self.tracker.codepoint_utf16_bit_array_segment_used {
+            self.register_prelude_usage(&mut imports, "codepointToUtf16", None);
+        }
+
+        if self.tracker.codepoint_utf32_bit_array_segment_used {
+            self.register_prelude_usage(&mut imports, "codepointToUtf32", None);
+        }
 
         if self.tracker.float_bit_array_segment_used {
             self.register_prelude_usage(&mut imports, "sizedFloat", None);
-        };
+        }
 
-        let echo = if self.tracker.echo_used {
-            if StdlibPackage::Present == self.stdlib_package {
-                self.register_import(
-                    &mut imports,
-                    "gleam_stdlib",
-                    "dict",
-                    &Some((
-                        AssignName::Variable("stdlib$dict".into()),
-                        SrcSpan::default(),
-                    )),
-                    &[],
-                );
-            }
-            self.register_prelude_usage(&mut imports, "BitArray", Some("$BitArray"));
-            self.register_prelude_usage(&mut imports, "List", Some("$List"));
-            self.register_prelude_usage(&mut imports, "UtfCodepoint", Some("$UtfCodepoint"));
-            docvec![line(), std::include_str!("../templates/echo.mjs"), line()]
-        } else {
-            nil()
-        };
+        let echo_definition = self.echo_definition(&mut imports);
+        let type_reference = self.type_reference();
+        let filepath_definition = self.filepath_definition();
 
         // Put it all together
 
         if imports.is_empty() && statements.is_empty() {
-            Ok(docvec![type_reference, "export {}", line(), echo])
+            Ok(docvec![
+                type_reference,
+                filepath_definition,
+                "export {}",
+                line(),
+                echo_definition
+            ])
         } else if imports.is_empty() {
             statements.push(line());
-            Ok(docvec![type_reference, statements, echo])
+            Ok(docvec![
+                type_reference,
+                filepath_definition,
+                statements,
+                echo_definition
+            ])
         } else if statements.is_empty() {
             Ok(docvec![
                 type_reference,
                 imports.into_doc(JavaScriptCodegenTarget::JavaScript),
-                echo,
+                filepath_definition,
+                echo_definition,
             ])
         } else {
             Ok(docvec![
                 type_reference,
                 imports.into_doc(JavaScriptCodegenTarget::JavaScript),
                 line(),
+                filepath_definition,
                 statements,
                 line(),
-                echo
+                echo_definition
             ])
         }
     }
 
+    fn echo_definition(&mut self, imports: &mut Imports<'a>) -> Document<'a> {
+        if !self.tracker.echo_used {
+            return nil();
+        }
+
+        if StdlibPackage::Present == self.stdlib_package {
+            let value = Some((
+                AssignName::Variable("stdlib$dict".into()),
+                SrcSpan::default(),
+            ));
+            self.register_import(imports, "gleam_stdlib", "dict", &value, &[]);
+        }
+        self.register_prelude_usage(imports, "BitArray", Some("$BitArray"));
+        self.register_prelude_usage(imports, "List", Some("$List"));
+        self.register_prelude_usage(imports, "UtfCodepoint", Some("$UtfCodepoint"));
+        docvec![line(), std::include_str!("../templates/echo.mjs"), line()]
+    }
+
     fn register_prelude_usage(
         &self,
         imports: &mut Imports<'a>,
@@ -316,7 +361,7 @@ impl<'a> Generator<'a> {
             arg.label
                 .as_ref()
                 .map(|(_, s)| maybe_escape_identifier(s))
-                .unwrap_or_else(|| eco_format!("x{i}"))
+                .unwrap_or_else(|| eco_format!("${i}"))
                 .to_doc()
         }
 
@@ -342,7 +387,7 @@ impl<'a> Generator<'a> {
                 match &arg.label {
                     None => docvec!["this[", i, "] = ", var, ";"],
                     Some((_, name)) => {
-                        docvec!["this.", maybe_escape_property_doc(name), " = ", var, ";"]
+                        docvec!["this.", maybe_escape_property(name), " = ", var, ";"]
                     }
                 }
             }),
@@ -515,8 +560,17 @@ impl<'a> Generator<'a> {
             "export const "
         };
 
-        let document =
-            expression::constant_expression(Context::Constant, &mut self.tracker, value)?;
+        let mut generator = expression::Generator::new(
+            self.module.name.clone(),
+            self.src_path.clone(),
+            self.line_numbers,
+            "".into(),
+            vec![],
+            &mut self.tracker,
+            self.module_scope.clone(),
+        );
+
+        let document = generator.constant_expression(Context::Constant, value)?;
 
         Ok(docvec![
             head,
@@ -543,8 +597,7 @@ impl<'a> Generator<'a> {
             .collect();
         let mut generator = expression::Generator::new(
             self.module.name.clone(),
-            &self.module.type_info.src_path,
-            self.project_root,
+            self.src_path.clone(),
             self.line_numbers,
             name.clone(),
             argument_names,
@@ -609,6 +662,14 @@ impl<'a> Generator<'a> {
             }
         }
     }
+
+    fn filepath_definition(&self) -> Document<'a> {
+        if !self.tracker.make_error_used {
+            return nil();
+        }
+
+        docvec!["const FILEPATH = ", self.src_path.clone(), ';', lines(2)]
+    }
 }
 
 #[derive(Debug)]
@@ -781,18 +842,18 @@ fn is_usable_js_identifier(word: &str) -> bool {
 }
 
 fn is_usable_js_property(label: &str) -> bool {
-    !matches!(
-        label,
-        // `then` to avoid a custom type that defines a `then` function being used as a `thenable`
-        // in Javascript.
+    match label {
+        // `then` to avoid a custom type that defines a `then` function being
+        // used as a `thenable` in Javascript.
         "then"
-            // `constructor` to avoid unintentional overriding of the constructor of records,
-            // leading to potential runtime crashes while using `withFields`.
-            | "constructor"
-            // `prototype` and `__proto__` to avoid unintentionally overriding the prototype chain
-            | "prototype"
-            | "__proto__"
-    )
+        // `constructor` to avoid unintentional overriding of the constructor of
+        // records, leading to potential runtime crashes while using `withFields`.
+        | "constructor"
+        // `prototype` and `__proto__` to avoid unintentionally overriding the
+        // prototype chain.
+        | "prototpye" | "__proto__" => false,
+        _ => true
+    }
 }
 
 fn maybe_escape_identifier_string(word: &str) -> EcoString {
@@ -815,11 +876,11 @@ fn maybe_escape_identifier(word: &str) -> EcoString {
     }
 }
 
-fn maybe_escape_property_doc(label: &str) -> Document<'_> {
+fn maybe_escape_property(label: &str) -> EcoString {
     if is_usable_js_property(label) {
-        label.to_doc()
+        EcoString::from(label)
     } else {
-        escape_identifier(label).to_doc()
+        escape_identifier(label)
     }
 }
 
@@ -827,6 +888,8 @@ fn maybe_escape_property_doc(label: &str) -> Document<'_> {
 pub(crate) struct UsageTracker {
     pub ok_used: bool,
     pub list_used: bool,
+    pub list_empty_class_used: bool,
+    pub list_non_empty_class_used: bool,
     pub prepend_used: bool,
     pub error_used: bool,
     pub int_remainder_used: bool,
@@ -841,7 +904,11 @@ pub(crate) struct UsageTracker {
     pub bit_array_slice_to_int_used: bool,
     pub sized_integer_segment_used: bool,
     pub string_bit_array_segment_used: bool,
+    pub string_utf16_bit_array_segment_used: bool,
+    pub string_utf32_bit_array_segment_used: bool,
     pub codepoint_bit_array_segment_used: bool,
+    pub codepoint_utf16_bit_array_segment_used: bool,
+    pub codepoint_utf32_bit_array_segment_used: bool,
     pub float_bit_array_segment_used: bool,
     pub echo_used: bool,
 }
