@@ -1,5 +1,5 @@
+use crate::assert_js;
 use crate::javascript::tests::CURRENT_PACKAGE;
-use crate::{assert_js, assert_js_with_multiple_imports};
 
 #[test]
 fn empty_module() {
@@ -10,7 +10,7 @@ fn empty_module() {
 #[test]
 fn unqualified_fn_call() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub fn launch() { 1 }"#),
+        ("rocket_ship", r#"pub fn launch() { 1 }"#),
         r#"import rocket_ship.{launch}
 pub fn go() { launch() }
 "#,
@@ -20,7 +20,7 @@ pub fn go() { launch() }
 #[test]
 fn aliased_unqualified_fn_call() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub fn launch() { 1 }"#),
+        ("rocket_ship", r#"pub fn launch() { 1 }"#),
         r#"import rocket_ship.{launch as boom_time}
 pub fn go() { boom_time() }
 "#,
@@ -46,7 +46,7 @@ pub fn go() { a() + bb() }
 #[test]
 fn constant() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub const x = 1"#),
+        ("rocket_ship", r#"pub const x = 1"#),
         r#"
 import rocket_ship
 pub fn go() { rocket_ship.x }
@@ -57,7 +57,7 @@ pub fn go() { rocket_ship.x }
 #[test]
 fn alias_aliased_constant() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub const x = 1"#),
+        ("rocket_ship", r#"pub const x = 1"#),
         r#"
 import rocket_ship.{ x as y }
 const z = y
@@ -68,7 +68,7 @@ const z = y
 #[test]
 fn renamed_module() {
     assert_js!(
-        (CURRENT_PACKAGE, "x", r#"pub const v = 1"#),
+        ("x", r#"pub const v = 1"#),
         r#"
 import x as y
 const z = y.v
@@ -94,7 +94,7 @@ pub fn go() { launcher.x }
 #[test]
 fn alias_constant() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub const x = 1"#),
+        ("rocket_ship", r#"pub const x = 1"#),
         r#"
 import rocket_ship as boop
 pub fn go() { boop.x }
@@ -105,7 +105,7 @@ pub fn go() { boop.x }
 #[test]
 fn alias_fn_call() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub fn go() { 1 }"#),
+        ("rocket_ship", r#"pub fn go() { 1 }"#),
         r#"
 import rocket_ship as boop
 pub fn go() { boop.go() }
@@ -116,7 +116,7 @@ pub fn go() { boop.go() }
 #[test]
 fn nested_fn_call() {
     assert_js!(
-        (CURRENT_PACKAGE, "one/two", r#"pub fn go() { 1 }"#),
+        ("one/two", r#"pub fn go() { 1 }"#),
         r#"import one/two
 pub fn go() { two.go() }"#,
     );
@@ -125,7 +125,7 @@ pub fn go() { two.go() }"#,
 #[test]
 fn nested_nested_fn_call() {
     assert_js!(
-        (CURRENT_PACKAGE, "one/two/three", r#"pub fn go() { 1 }"#),
+        ("one/two/three", r#"pub fn go() { 1 }"#),
         r#"import one/two/three
 pub fn go() { three.go() }"#,
     );
@@ -144,7 +144,7 @@ pub fn go() { one.go() }
 #[test]
 fn nested_same_package() {
     assert_js!(
-        (CURRENT_PACKAGE, "one/two/three", r#"pub fn go() { 1 }"#),
+        ("one/two/three", r#"pub fn go() { 1 }"#),
         r#"import one/two/three
 pub fn go() { three.go() }
 "#,
@@ -153,9 +153,9 @@ pub fn go() { three.go() }
 
 #[test]
 fn discarded_duplicate_import() {
-    assert_js_with_multiple_imports!(
+    assert_js!(
         ("esa/rocket_ship", r#"pub fn go() { 1 }"#),
-        ("nasa/rocket_ship", r#"pub fn go() { 1 }"#);
+        ("nasa/rocket_ship", r#"pub fn go() { 1 }"#),
         r#"
 import esa/rocket_ship
 import nasa/rocket_ship as _nasa_rocket
@@ -166,9 +166,9 @@ pub fn go() { rocket_ship.go() }
 
 #[test]
 fn discarded_duplicate_import_with_unqualified() {
-    assert_js_with_multiple_imports!(
+    assert_js!(
         ("esa/rocket_ship", r#"pub fn go() { 1 }"#),
-        ("nasa/rocket_ship", r#"pub fn go() { 1 }"#);
+        ("nasa/rocket_ship", r#"pub fn go() { 1 }"#),
         r#"
 import esa/rocket_ship
 import nasa/rocket_ship.{go} as _nasa_rocket
@@ -202,7 +202,7 @@ pub fn main() {
 #[test]
 fn constant_module_access_with_keyword() {
     assert_js!(
-        (CURRENT_PACKAGE, "rocket_ship", r#"pub const class = 1"#),
+        ("rocket_ship", r#"pub const class = 1"#),
         r#"
 import rocket_ship
 pub const variable = rocket_ship.class
