@@ -14,9 +14,9 @@ use crate::{
     Result,
     ast::{
         self, Arg, CallArg, Definition, Function, FunctionLiteralKind, Pattern, Publicity,
-        TypedExpr,
+        TypedExpr, visit::Visit,
     },
-    build::Module,
+    build::{Module, Origin},
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     line_numbers::LineNumbers,
     type_::{
@@ -130,9 +130,7 @@ where
         &'a self,
         valid_phrase_char: &impl Fn(char) -> bool,
     ) -> (Range, String) {
-        let cursor = self
-            .src_line_numbers
-            .byte_index(self.cursor_position.line, self.cursor_position.character);
+        let cursor = self.src_line_numbers.byte_index(*self.cursor_position);
 
         // Get part of phrase prior to cursor
         let before = self
@@ -193,12 +191,14 @@ where
     /// If the line includes a dot then it provides unqualified import completions.
     /// Otherwise it provides direct module import completions.
     pub fn import_completions(&'a self) -> Option<Result<Option<Vec<CompletionItem>>>> {
-        let start_of_line = self
-            .src_line_numbers
-            .byte_index(self.cursor_position.line, 0);
-        let end_of_line = self
-            .src_line_numbers
-            .byte_index(self.cursor_position.line + 1, 0);
+        let start_of_line = self.src_line_numbers.byte_index(Position {
+            line: self.cursor_position.line,
+            character: 0,
+        });
+        let end_of_line = self.src_line_numbers.byte_index(Position {
+            line: self.cursor_position.line + 1,
+            character: 0,
+        });
 
         // Drop all lines except the line the cursor is on
         let src = self.src.get(start_of_line as usize..end_of_line as usize)?;
@@ -347,16 +347,18 @@ where
             .get_importable_modules()
             .iter()
             //
-            // It is possible to import modules from dependencies of dependencies
-            // but it's not recommended so we don't include them in completions
-            .filter(|(_, module)| {
-                let is_root_or_prelude =
-                    module.package == self.root_package_name() || module.package.is_empty();
-                is_root_or_prelude || direct_dep_packages.contains(&module.package)
-            })
+            // You cannot import yourself
+            .filter(|(name, _)| *name != &self.module.name)
             //
-            // src/ cannot import test/
-            .filter(|(_, module)| module.origin.is_src() || !self.module.origin.is_src())
+            // Different origin directories will get different import completions
+            .filter(|(_, module)| match self.module.origin {
+                // src/ can import from src/
+                Origin::Src => module.origin.is_src(),
+                // dev/ can import from src/ or dev/
+                Origin::Dev => !module.origin.is_test(),
+                // Test can import from anywhere
+                Origin::Test => true,
+            })
             //
             // It is possible to import internal modules from other packages,
             // but it's not recommended so we don't include them in completions
@@ -365,8 +367,13 @@ where
             // You cannot import a module twice
             .filter(|(name, _)| !already_imported.contains(*name))
             //
-            // You cannot import yourself
-            .filter(|(name, _)| *name != &self.module.name)
+            // It is possible to import modules from dependencies of dependencies
+            // but it's not recommended so we don't include them in completions
+            .filter(|(_, module)| {
+                let is_root_or_prelude =
+                    module.package == self.root_package_name() || module.package.is_empty();
+                is_root_or_prelude || direct_dep_packages.contains(&module.package)
+            })
             .collect()
     }
 
@@ -554,9 +561,7 @@ where
         // e.x. when the user has typed mymodule.| we know local module and prelude values are no longer
         // relevant.
         if module_select.is_none() {
-            let cursor = self
-                .src_line_numbers
-                .byte_index(self.cursor_position.line, self.cursor_position.character);
+            let cursor = self.src_line_numbers.byte_index(*self.cursor_position);
 
             // Find the function that the cursor is in and push completions for
             // its arguments and local variables.
@@ -1013,14 +1018,8 @@ impl<'a> LocalCompletion<'a> {
 
         // Visit the function body statements
         for statement in &fun.body {
-            // We only want to suggest local variables that are defined before
-            // the cursor
-            if statement.location().start >= self.cursor {
-                continue;
-            }
-
             // Visit the statement to find local variables
-            ast::visit::visit_typed_statement(&mut self, statement);
+            self.visit_typed_statement(statement);
         }
 
         self.completions.into_values().collect_vec()
@@ -1046,7 +1045,16 @@ impl<'a> LocalCompletion<'a> {
     }
 }
 
-impl<'ast> ast::visit::Visit<'ast> for LocalCompletion<'_> {
+impl<'ast> Visit<'ast> for LocalCompletion<'_> {
+    fn visit_typed_statement(&mut self, statement: &'ast ast::TypedStatement) {
+        // We only want to suggest local variables that are defined before
+        // the cursor
+        if statement.location().start >= self.cursor {
+            return;
+        }
+        ast::visit::visit_typed_statement(self, statement);
+    }
+
     /// Visits a typed assignment, selectively processing either the value or the pattern
     /// based on the cursor position.
     /// - If the cursor is within the assignment It visits only the value expression.
@@ -1063,19 +1071,46 @@ impl<'ast> ast::visit::Visit<'ast> for LocalCompletion<'_> {
 
     fn visit_typed_expr_fn(
         &mut self,
-        _: &'ast ast::SrcSpan,
+        location: &'ast ast::SrcSpan,
         _: &'ast Arc<Type>,
         _: &'ast FunctionLiteralKind,
         args: &'ast [ast::TypedArg],
         body: &'ast Vec1<ast::TypedStatement>,
         _: &'ast Option<ast::TypeAst>,
     ) {
+        // If we are completing after the function body, any locally defined
+        // variables are now out of scope so we don't register any.
+        if self.cursor >= location.end {
+            return;
+        }
         self.visit_fn_args(args);
         for statement in body {
             self.visit_typed_statement(statement);
         }
     }
 
+    fn visit_typed_expr_block(
+        &mut self,
+        location: &'ast ast::SrcSpan,
+        statements: &'ast [ast::TypedStatement],
+    ) {
+        // If we are completing after the block, any locally defined variables
+        // are now out of scope so we don't register any.
+        if self.cursor >= location.end {
+            return;
+        }
+        ast::visit::visit_typed_expr_block(self, location, statements);
+    }
+
+    fn visit_typed_clause(&mut self, clause: &'ast ast::TypedClause) {
+        // Any code which comes before or after a case clause cannot access any
+        // of the variables defined within it, so we ignore this clause if so.
+        if self.cursor < clause.location.start || self.cursor > clause.location.end {
+            return;
+        }
+        ast::visit::visit_typed_clause(self, clause);
+    }
+
     fn visit_typed_pattern_variable(
         &mut self,
         _: &'ast ast::SrcSpan,
