@@ -11,14 +11,15 @@ use crate::{
         TypedConstantBitArraySegment, TypedConstantBitArraySegmentOption,
     },
     build::Origin,
-    line_numbers::LineNumbers,
+    line_numbers::{Character, LineNumbers},
     reference::{Reference, ReferenceKind, ReferenceMap},
     schema_capnp::{self as schema, *},
     type_::{
         self, AccessorsMap, Deprecation, FieldMap, ModuleInterface, Opaque, RecordAccessor,
         References, Type, TypeAliasConstructor, TypeConstructor, TypeValueConstructor,
         TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
-        ValueConstructorVariant, expression::Implementations,
+        ValueConstructorVariant,
+        expression::{Implementations, Purity},
     },
     uid::UniqueIdGenerator,
 };
@@ -431,6 +432,7 @@ impl ModuleDecoder {
             tag,
             type_,
             field_map: None,
+            record_constructor: None,
         })
     }
 
@@ -601,6 +603,13 @@ impl ModuleDecoder {
         &self,
         reader: &value_constructor_variant::module_fn::Reader<'_>,
     ) -> Result<ValueConstructorVariant> {
+        let purity = match reader.get_purity()?.which()? {
+            purity::Which::Pure(()) => Purity::Pure,
+            purity::Which::TrustedPure(()) => Purity::TrustedPure,
+            purity::Which::Impure(()) => Purity::Impure,
+            purity::Which::Unknown(()) => Purity::Unknown,
+        };
+
         Ok(ValueConstructorVariant::ModuleFn {
             name: self.string(reader.get_name()?)?,
             module: self.string(reader.get_module()?)?,
@@ -611,6 +620,7 @@ impl ModuleDecoder {
             implementations: self.implementations(reader.get_implementations()?),
             external_erlang: self.optional_external(reader.get_external_erlang()?)?,
             external_javascript: self.optional_external(reader.get_external_javascript()?)?,
+            purity,
         })
     }
 
@@ -698,9 +708,30 @@ impl ModuleDecoder {
         Ok(LineNumbers {
             length: reader.get_length(),
             line_starts: read_vec!(reader.get_line_starts()?, self, line_starts),
+            mapping: self.mapping(reader.get_mapping()?),
         })
     }
 
+    fn mapping(
+        &self,
+        reader: capnp::struct_list::Reader<'_, character::Owned>,
+    ) -> HashMap<usize, Character> {
+        let mut map = HashMap::with_capacity(reader.len() as usize);
+        for character in reader.into_iter() {
+            let byte_index = character.get_byte_index() as usize;
+            let length_utf8 = character.get_length_utf8();
+            let length_utf16 = character.get_length_utf16();
+            _ = map.insert(
+                byte_index,
+                Character {
+                    length_utf16,
+                    length_utf8,
+                },
+            )
+        }
+        map
+    }
+
     fn version(&self, reader: &version::Reader<'_>) -> hexpm::version::Version {
         hexpm::version::Version::new(reader.get_major(), reader.get_minor(), reader.get_patch())
     }
