@@ -11,7 +11,7 @@ use crate::{
         self, AccessorsMap, Deprecation, FieldMap, Opaque, RecordAccessor, Type,
         TypeAliasConstructor, TypeConstructor, TypeValueConstructor, TypeVar,
         TypeVariantConstructors, ValueConstructor, ValueConstructorVariant,
-        expression::Implementations,
+        expression::{Implementations, Purity},
     },
 };
 use std::{collections::HashMap, ops::Deref, sync::Arc};
@@ -62,11 +62,21 @@ impl<'a> ModuleEncoder<'a> {
     fn set_line_numbers(&mut self, module: &mut module::Builder<'_>) {
         let mut line_numbers = module.reborrow().init_line_numbers();
         line_numbers.set_length(self.data.line_numbers.length);
-        let mut line_starts =
-            line_numbers.init_line_starts(self.data.line_numbers.line_starts.len() as u32);
+
+        let mut line_starts = line_numbers
+            .reborrow()
+            .init_line_starts(self.data.line_numbers.line_starts.len() as u32);
         for (i, l) in self.data.line_numbers.line_starts.iter().enumerate() {
             line_starts.reborrow().set(i as u32, *l);
         }
+
+        let mut mapping = line_numbers.init_mapping(self.data.line_numbers.mapping.len() as u32);
+        for (i, (byte_index, character)) in self.data.line_numbers.mapping.iter().enumerate() {
+            let mut builder = mapping.reborrow().get(i as u32);
+            builder.set_byte_index(*byte_index as u64);
+            builder.set_length_utf8(character.length_utf8);
+            builder.set_length_utf16(character.length_utf16);
+        }
     }
 
     fn set_module_documentation(&mut self, module: &mut module::Builder<'_>) {
@@ -430,12 +440,22 @@ impl<'a> ModuleEncoder<'a> {
                 implementations,
                 external_erlang,
                 external_javascript,
+                purity,
             } => {
                 let mut builder = builder.init_module_fn();
                 builder.set_name(name);
                 builder.set_module(module);
                 builder.set_arity(*arity as u16);
                 builder.set_documentation(doc.as_ref().map(EcoString::as_str).unwrap_or_default());
+
+                let mut purity_builder = builder.reborrow().init_purity();
+                match purity {
+                    Purity::Pure => purity_builder.set_pure(()),
+                    Purity::TrustedPure => purity_builder.set_trusted_pure(()),
+                    Purity::Impure => purity_builder.set_impure(()),
+                    Purity::Unknown => purity_builder.set_unknown(()),
+                }
+
                 self.build_external(builder.reborrow().init_external_erlang(), external_erlang);
                 self.build_external(
                     builder.reborrow().init_external_javascript(),
