@@ -14,7 +14,9 @@ use crate::{
     type_::{
         self, Deprecation, ModuleInterface, Opaque, References, Type, TypeAliasConstructor,
         TypeConstructor, TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors,
-        ValueConstructor, ValueConstructorVariant, expression::Implementations, prelude,
+        ValueConstructor, ValueConstructorVariant,
+        expression::{Implementations, Purity},
+        prelude,
     },
     uid::UniqueIdGenerator,
 };
@@ -528,6 +530,7 @@ fn module_fn_value() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Pure,
                 },
             },
         )]
@@ -582,6 +585,7 @@ fn deprecated_module_fn_value() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Pure,
                 },
             },
         )]
@@ -634,6 +638,7 @@ fn private_module_fn_value() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Pure,
                 },
             },
         )]
@@ -688,6 +693,7 @@ fn module_fn_value_regression() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::TrustedPure,
                 },
             },
         )]
@@ -741,6 +747,7 @@ fn module_fn_value_with_field_map() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Pure,
                 },
             },
         )]
@@ -760,7 +767,7 @@ fn module_fn_value_with_field_map() {
 
 #[test]
 fn record_value() {
-    let mut random = rand::thread_rng();
+    let mut random = rand::rng();
 
     let module = ModuleInterface {
         warnings: vec![],
@@ -782,13 +789,13 @@ fn record_value() {
                     name: "one".into(),
                     module: "themodule".into(),
                     field_map: None,
-                    arity: random.r#gen(),
-                    variants_count: random.r#gen(),
+                    arity: random.random(),
+                    variants_count: random.random(),
                     location: SrcSpan {
-                        start: random.r#gen(),
-                        end: random.r#gen(),
+                        start: random.random(),
+                        end: random.random(),
                     },
-                    variant_index: random.r#gen(),
+                    variant_index: random.random(),
                 },
             },
         )]
@@ -808,7 +815,7 @@ fn record_value() {
 
 #[test]
 fn record_value_with_field_map() {
-    let mut random = rand::thread_rng();
+    let mut random = rand::rng();
 
     let module = ModuleInterface {
         warnings: vec![],
@@ -830,16 +837,19 @@ fn record_value_with_field_map() {
                     module: "themodule".into(),
                     name: "one".into(),
                     field_map: Some(FieldMap {
-                        arity: random.r#gen(),
-                        fields: [("ok".into(), random.r#gen()), ("ko".into(), random.r#gen())]
-                            .into(),
+                        arity: random.random(),
+                        fields: [
+                            ("ok".into(), random.random()),
+                            ("ko".into(), random.random()),
+                        ]
+                        .into(),
                     }),
-                    arity: random.r#gen(),
-                    variants_count: random.r#gen(),
-                    variant_index: random.r#gen(),
+                    arity: random.random(),
+                    variants_count: random.random(),
+                    variant_index: random.random(),
                     location: SrcSpan {
-                        start: random.r#gen(),
-                        end: random.r#gen(),
+                        start: random.random(),
+                        end: random.random(),
                     },
                 },
             },
@@ -1125,6 +1135,7 @@ fn constant_record() {
         tag: "thetag".into(),
         type_: type_::int(),
         field_map: None,
+        record_constructor: None,
     });
 
     assert_eq!(roundtrip(&module), module);
@@ -1465,6 +1476,7 @@ fn module_fn_value_with_external_implementations() {
                         can_run_on_erlang: false,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Impure,
                 },
             },
         )]
@@ -1543,6 +1555,7 @@ fn internal_module_fn() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Unknown,
                 },
             },
         )]
@@ -1601,6 +1614,7 @@ fn internal_annotated_module_fn() {
                         can_run_on_erlang: true,
                         can_run_on_javascript: true,
                     },
+                    purity: Purity::Impure,
                 },
             },
         )]
