@@ -12,9 +12,28 @@ use crate::{
 use super::*;
 use std::collections::HashMap;
 
+#[derive(Debug)]
+pub struct EnvironmentArguments<'a> {
+    pub ids: UniqueIdGenerator,
+    pub current_package: EcoString,
+    pub gleam_version: Option<Range<Version>>,
+    pub current_module: EcoString,
+    pub target: Target,
+    pub importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
+    pub target_support: TargetSupport,
+    pub current_origin: Origin,
+}
+
+impl<'a> EnvironmentArguments<'a> {
+    pub fn build(self) -> Environment<'a> {
+        Environment::new(self)
+    }
+}
+
 #[derive(Debug)]
 pub struct Environment<'a> {
     pub current_package: EcoString,
+    pub origin: Origin,
 
     /// The gleam version range required by the current package as stated in its
     /// gleam.toml
@@ -71,13 +90,16 @@ pub struct Environment<'a> {
 
 impl<'a> Environment<'a> {
     pub fn new(
-        ids: UniqueIdGenerator,
-        current_package: EcoString,
-        gleam_version: Option<Range<Version>>,
-        current_module: EcoString,
-        target: Target,
-        importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
-        target_support: TargetSupport,
+        EnvironmentArguments {
+            ids,
+            current_package,
+            gleam_version,
+            current_module,
+            target,
+            importable_modules,
+            target_support,
+            current_origin: origin,
+        }: EnvironmentArguments<'a>,
     ) -> Self {
         let prelude = importable_modules
             .get(PRELUDE_MODULE_NAME)
@@ -101,6 +123,7 @@ impl<'a> Environment<'a> {
             gleam_version,
             previous_id: ids.next(),
             ids,
+            origin,
             target,
             module_types: prelude.types.clone(),
             module_types_constructors: prelude.types_value_constructors.clone(),
@@ -206,12 +229,7 @@ impl Environment<'_> {
     ) {
         let _ = self.scope.insert(
             name,
-            ValueConstructor {
-                deprecation: Deprecation::NotDeprecated,
-                publicity: Publicity::Private,
-                variant: ValueConstructorVariant::LocalVariable { location, origin },
-                type_,
-            },
+            ValueConstructor::local_variable(location, origin, type_),
         );
     }
 
@@ -354,10 +372,10 @@ impl Environment<'_> {
     ///
     pub fn get_type_constructor(
         &mut self,
-        module_alias: &Option<(EcoString, SrcSpan)>,
+        module: &Option<(EcoString, SrcSpan)>,
         name: &EcoString,
     ) -> Result<&TypeConstructor, UnknownTypeConstructorError> {
-        match module_alias {
+        match module {
             None => self
                 .module_types
                 .get(name)
@@ -627,6 +645,22 @@ impl Environment<'_> {
             .expect("Expected a bottom level of entity usages.");
         self.handle_unused_variables(unused, problems);
 
+        // We have to handle unused imported entites a bit differently when
+        // emitting warning: when an import list is unused all its items and
+        // the import itself are unused:
+        //
+        // ```
+        // import wibble.{unused, also_unused}
+        //        ^^^^^^  ^^^^^^  ^^^^^^^^^^^ Everything is unused here
+        // ```
+        //
+        // But instead of emitting three warnings, what we really want is to
+        // emit just a single warning encompassing the entire line! So we have
+        // to hold on all unused imported entities and emit a warning for those
+        // only if the module they come from is not also unused.
+        let mut unused_modules = HashSet::new();
+        let mut unused_imported_items = vec![];
+
         for (entity, info) in self.references.unused() {
             let name = entity.name;
             let location = info.origin;
@@ -639,37 +673,61 @@ impl Environment<'_> {
                     name,
                     imported: false,
                 },
-                EntityKind::ImportedType => Warning::UnusedType {
-                    name,
-                    imported: true,
-                    location,
-                },
-                EntityKind::ImportedConstructor => Warning::UnusedConstructor {
-                    name,
-                    imported: true,
-                    location,
-                },
                 EntityKind::Type => Warning::UnusedType {
                     name,
                     imported: false,
                     location,
                 },
-                EntityKind::ImportedValue => Warning::UnusedImportedValue { name, location },
-                EntityKind::ImportedModule => Warning::UnusedImportedModule { name, location },
-                EntityKind::ModuleAlias => {
-                    let module_name = match self.imported_modules.get(&name) {
-                        Some((_, module)) => module.name.clone(),
-                        None => name.clone(),
-                    };
-                    Warning::UnusedImportedModuleAlias {
-                        module_name,
-                        alias: name,
-                        location,
-                    }
+                EntityKind::ImportedModule { module_name } => {
+                    let _ = unused_modules.insert(module_name.clone());
+                    Warning::UnusedImportedModule { name, location }
+                }
+                EntityKind::ImportedType { module } => {
+                    unused_imported_items.push((
+                        module,
+                        Warning::UnusedType {
+                            name,
+                            imported: true,
+                            location,
+                        },
+                    ));
+                    continue;
+                }
+                EntityKind::ImportedConstructor { module } => {
+                    unused_imported_items.push((
+                        module,
+                        Warning::UnusedConstructor {
+                            name,
+                            imported: true,
+                            location,
+                        },
+                    ));
+                    continue;
+                }
+                EntityKind::ImportedValue { module } => {
+                    unused_imported_items
+                        .push((module, Warning::UnusedImportedValue { name, location }));
+                    continue;
+                }
+                EntityKind::ModuleAlias { module } => {
+                    unused_imported_items.push((
+                        module.clone(),
+                        Warning::UnusedImportedModuleAlias {
+                            module_name: module.clone(),
+                            alias: name,
+                            location,
+                        },
+                    ));
+                    continue;
                 }
             };
             problems.warning(warning);
         }
+
+        unused_imported_items
+            .into_iter()
+            .filter(|(module, _)| !unused_modules.contains(module))
+            .for_each(|(_, warning)| problems.warning(warning));
     }
 
     fn handle_unused_variables(
