@@ -30,6 +30,7 @@ mod externals;
 mod functions;
 mod guards;
 mod imports;
+mod let_assert;
 mod pipes;
 mod pretty;
 mod target_implementations;
@@ -145,7 +146,7 @@ macro_rules! assert_error {
         let (error, names) = $crate::type_::tests::compile_statement_sequence($src)
             .expect_err("should infer an error");
         let error = $crate::error::Error::Type {
-            names,
+            names: Box::new(names),
             src: $src.into(),
             path: camino::Utf8PathBuf::from("/src/one/two.gleam"),
             errors: error,
@@ -232,7 +233,12 @@ macro_rules! assert_warning {
             None
         );
         assert!(!warning.is_empty());
-        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
+
+        let mut output = String::from("----- SOURCE CODE\n");
+        for (name, src) in [$(($name, $module_src)),*] {
+            output.push_str(&format!("-- {name}.gleam\n{src}\n\n"));
+        }
+        output.push_str(&format!("-- main.gleam\n{}\n\n----- WARNING\n{warning}", $src));
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -314,6 +320,15 @@ macro_rules! assert_no_warnings {
         let warnings = $crate::type_::tests::get_warnings($src, vec![], crate::build::Target::Erlang, None);
         assert_eq!(warnings, vec![]);
     };
+    ($(($name:expr, $module_src:literal)),+, $src:expr $(,)?) => {
+        let warnings = $crate::type_::tests::get_warnings(
+            $src,
+            vec![$(("thepackage", $name, $module_src)),*],
+            crate::build::Target::Erlang,
+            None,
+        );
+        assert_eq!(warnings, vec![]);
+    };
     ($(($package:expr, $name:expr, $module_src:literal)),+, $src:expr $(,)?) => {
         let warnings = $crate::type_::tests::get_warnings(
             $src,
@@ -337,15 +352,17 @@ fn compile_statement_sequence(
     // place.
     let _ = modules.insert(PRELUDE_MODULE_NAME.into(), build_prelude(&ids));
     let mut problems = Problems::new();
-    let mut environment = Environment::new(
+    let mut environment = EnvironmentArguments {
         ids,
-        "thepackage".into(),
-        None,
-        "themodule".into(),
-        Target::Erlang,
-        &modules,
-        TargetSupport::Enforced,
-    );
+        current_package: "thepackage".into(),
+        gleam_version: None,
+        current_module: "themodule".into(),
+        target: Target::Erlang,
+        importable_modules: &modules,
+        target_support: TargetSupport::Enforced,
+        current_origin: Origin::Src,
+    }
+    .build();
     let res = ExprTyper::new(
         &mut environment,
         FunctionDefinition {
@@ -530,7 +547,7 @@ pub fn module_error_with_target(
     )
     .expect_err("should infer an error");
     let error = Error::Type {
-        names,
+        names: Box::new(names),
         src: src.into(),
         path: Utf8PathBuf::from("/src/one/two.gleam"),
         errors: Vec1::try_from_vec(error).expect("should have at least one error"),
@@ -558,7 +575,7 @@ pub fn internal_module_error_with_target(
     )
     .expect_err("should infer an error");
     let error = Error::Type {
-        names,
+        names: Box::new(names),
         src: src.into(),
         path: Utf8PathBuf::from("/src/one/two.gleam"),
         errors: Vec1::try_from_vec(error).expect("should have at least one error"),
@@ -573,7 +590,7 @@ pub fn syntax_error(src: &str) -> String {
     let error = Error::Parse {
         src: src.into(),
         path: Utf8PathBuf::from("/src/one/two.gleam"),
-        error,
+        error: Box::new(error),
     };
     error.pretty_string()
 }
@@ -2678,7 +2695,10 @@ fn assert_suitable_main_function_not_module_function() {
             name: "main".into(),
         },
     };
-    assert!(assert_suitable_main_function(&value, &"module".into(), Target::Erlang).is_err(),);
+    assert!(
+        assert_suitable_main_function(&value, &"module".into(), Origin::Src, Target::Erlang)
+            .is_err(),
+    );
 }
 
 #[test]
@@ -2703,9 +2723,13 @@ fn assert_suitable_main_function_wrong_arity() {
                 can_run_on_erlang: true,
                 can_run_on_javascript: true,
             },
+            purity: Purity::Pure,
         },
     };
-    assert!(assert_suitable_main_function(&value, &"module".into(), Target::Erlang).is_err(),);
+    assert!(
+        assert_suitable_main_function(&value, &"module".into(), Origin::Src, Target::Erlang)
+            .is_err(),
+    );
 }
 
 #[test]
@@ -2730,9 +2754,13 @@ fn assert_suitable_main_function_ok() {
                 can_run_on_erlang: true,
                 can_run_on_javascript: true,
             },
+            purity: Purity::Pure,
         },
     };
-    assert!(assert_suitable_main_function(&value, &"module".into(), Target::Erlang).is_ok(),);
+    assert!(
+        assert_suitable_main_function(&value, &"module".into(), Origin::Src, Target::Erlang)
+            .is_ok(),
+    );
 }
 
 #[test]
@@ -2757,9 +2785,13 @@ fn assert_suitable_main_function_erlang_not_supported() {
                 can_run_on_erlang: false,
                 can_run_on_javascript: true,
             },
+            purity: Purity::Impure,
         },
     };
-    assert!(assert_suitable_main_function(&value, &"module".into(), Target::Erlang).is_err(),);
+    assert!(
+        assert_suitable_main_function(&value, &"module".into(), Origin::Src, Target::Erlang)
+            .is_err(),
+    );
 }
 
 #[test]
@@ -2784,9 +2816,13 @@ fn assert_suitable_main_function_javascript_not_supported() {
                 can_run_on_erlang: true,
                 can_run_on_javascript: false,
             },
+            purity: Purity::Impure,
         },
     };
-    assert!(assert_suitable_main_function(&value, &"module".into(), Target::JavaScript).is_err(),);
+    assert!(
+        assert_suitable_main_function(&value, &"module".into(), Origin::Src, Target::JavaScript)
+            .is_err(),
+    );
 }
 
 #[test]
@@ -3071,3 +3107,96 @@ pub type Bad {
 "
     );
 }
+
+#[test]
+fn function_parameter_errors_do_not_stop_inference() {
+    assert_module_error!(
+        "
+pub fn wibble(x: NonExistent) {
+  1 + False
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4287
+#[test]
+fn no_stack_overflow_for_nested_use() {
+    assert_module_infer!(
+        (
+            "gleam/dynamic/decode",
+            "
+pub fn string() {
+  Nil
+}
+
+pub fn field(name, decoder, callback) {
+  callback(Nil)
+}
+"
+        ),
+        r#"
+import gleam/dynamic/decode
+
+pub fn main() {
+  use _n1 <- decode.field("n1", decode.string)
+  use _n2 <- decode.field("n2", decode.string)
+  use _n3 <- decode.field("n3", decode.string)
+  use _n4 <- decode.field("n4", decode.string)
+  use _n5 <- decode.field("n5", decode.string)
+  use _n6 <- decode.field("n6", decode.string)
+  use _n7 <- decode.field("n7", decode.string)
+  use _n8 <- decode.field("n8", decode.string)
+  use _n9 <- decode.field("n9", decode.string)
+  use _n10 <- decode.field("n10", decode.string)
+  use _n11 <- decode.field("n11", decode.string)
+  use _n12 <- decode.field("n12", decode.string)
+  use _n13 <- decode.field("n13", decode.string)
+  use _n14 <- decode.field("n14", decode.string)
+  use _n15 <- decode.field("n15", decode.string)
+  use _n16 <- decode.field("n16", decode.string)
+  use _n17 <- decode.field("n17", decode.string)
+  use _n18 <- decode.field("n18", decode.string)
+  use _n19 <- decode.field("n19", decode.string)
+  use _n20 <- decode.field("n20", decode.string)
+  use _n21 <- decode.field("n21", decode.string)
+  use _n22 <- decode.field("n22", decode.string)
+  use _n23 <- decode.field("n23", decode.string)
+  use _n24 <- decode.field("n24", decode.string)
+  use _n25 <- decode.field("n25", decode.string)
+  use _n26 <- decode.field("n26", decode.string)
+  use _n27 <- decode.field("n27", decode.string)
+  use _n28 <- decode.field("n28", decode.string)
+  use _n29 <- decode.field("n29", decode.string)
+  use _n30 <- decode.field("n30", decode.string)
+  use _n31 <- decode.field("n31", decode.string)
+  use _n32 <- decode.field("n32", decode.string)
+  use _n33 <- decode.field("n33", decode.string)
+  use _n34 <- decode.field("n34", decode.string)
+  use _n35 <- decode.field("n35", decode.string)
+  use _n36 <- decode.field("n36", decode.string)
+  use _n37 <- decode.field("n37", decode.string)
+  use _n38 <- decode.field("n38", decode.string)
+  use _n39 <- decode.field("n39", decode.string)
+  use _n40 <- decode.field("n40", decode.string)
+  use _n41 <- decode.field("n41", decode.string)
+  use _n42 <- decode.field("n42", decode.string)
+  use _n43 <- decode.field("n43", decode.string)
+  use _n44 <- decode.field("n44", decode.string)
+  use _n45 <- decode.field("n45", decode.string)
+  use _n46 <- decode.field("n46", decode.string)
+  use _n47 <- decode.field("n47", decode.string)
+  use _n48 <- decode.field("n48", decode.string)
+  use _n49 <- decode.field("n49", decode.string)
+  use _n50 <- decode.field("n50", decode.string)
+  use _n51 <- decode.field("n51", decode.string)
+  use _n52 <- decode.field("n52", decode.string)
+  use _n53 <- decode.field("n53", decode.string)
+  use _n54 <- decode.field("n54", decode.string)
+  use _n55 <- decode.field("n55", decode.string)
+  Nil
+}
+"#,
+        vec![("main", "fn() -> Nil")]
+    );
+}
