@@ -164,6 +164,16 @@ fn bit_array_segment_type_does_not_allow_size_utf32() {
     assert_error!("case <<1>> { <<1:utf32-size(5)>> -> 1 }");
 }
 
+#[test]
+fn bit_array_segment_type_does_not_allow_variable_string() {
+    assert_error!("case <<>> { <<a:utf8>> -> 1 _ -> 2 }");
+}
+
+#[test]
+fn bit_array_segment_type_does_not_allow_aliased_variable_string() {
+    assert_error!("case <<>> { <<_ as a:utf8>> -> 1 _ -> 2 }");
+}
+
 #[test]
 fn bit_array_segment_unit_no_size() {
     assert_error!("let x = <<1:unit(5)>> x");
@@ -3016,3 +3026,102 @@ fn int_operator_on_floats_2() {
 fn add_on_strings() {
     assert_error!(r#""Hello, " + "Jak""#);
 }
+
+#[test]
+fn fault_tolerant_list() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  [1, "a", 1.0, "a" + 1]
+}
+"#
+    );
+}
+
+#[test]
+fn fault_tolerant_list_tail() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  [1, "a", ..["a", "b"]]
+}
+"#
+    );
+}
+
+#[test]
+fn fault_tolerant_negate_bool() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  !!{ True || a }
+}
+"#
+    );
+}
+
+#[test]
+fn fault_tolerant_negate_int() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  --{ 1 + a }
+}
+"#
+    );
+}
+
+#[test]
+fn fault_tolerant_tuple() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  #(1, 1 + "a", not_in_scope)
+}
+"#
+    );
+}
+
+#[test]
+fn error_for_missing_type_parameters() {
+    assert_module_error!(
+        r#"
+type Wibble(a)
+
+type Wobble {
+  Wobble(Wibble)
+}
+"#
+    );
+}
+
+#[test]
+fn double_assignment_in_bit_array() {
+    assert_error!("let assert <<a as b>> = <<>>");
+}
+
+#[test]
+fn negative_size_pattern() {
+    assert_error!("let assert <<1:size(-1)>> = <<>>");
+}
+
+#[test]
+fn zero_size_pattern() {
+    assert_error!("let assert <<1:size(0)>> = <<>>");
+}
+
+// https://github.com/gleam-lang/gleam/issues/3253
+#[test]
+fn bit_array_using_pattern_variables() {
+    assert_error!("let assert #(a, <<b:size(a)>>) = #(2, <<2:2>>)");
+}
+
+#[test]
+fn bit_array_using_pattern_variables_from_other_bit_array() {
+    assert_error!("let assert #(<<a>>, <<b:size(a)>>) = #(<<2>>, <<2:2>>)");
+}
+
+#[test]
+fn non_utf8_string_assignment() {
+    assert_error!(r#"let assert <<"Hello" as message:utf16>> = <<>>"#);
+}
