@@ -1483,3 +1483,307 @@ fn multiple_unreachable_prefix_patterns_1() {
 }"#
     );
 }
+
+#[test]
+fn bit_array_bits_catches_everything() {
+    assert_warning!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<_:bits>> -> 1
+    <<1>> -> 2
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_bytes_needs_catch_all() {
+    assert_module_error!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<_:bytes>> -> 1
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_overlapping_patterns_are_redundant() {
+    assert_warning!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<1, a:size(16)>> -> a
+    <<1, b:size(8)-unit(2)>> -> b
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_similar_overlapping_patterns_are_not_redundant() {
+    assert_no_warnings!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<1, a:size(16)>> -> a
+    <<2, b:size(8)-unit(2)>> -> b
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_overlapping_redundant_patterns_with_variable_size() {
+    assert_warning!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  let len = 3
+  case bit_array {
+    <<a:size(len), _:size(16)>> -> a
+    <<_:size(len), b:size(8)-unit(2)>> -> b
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_overlapping_redundant_patterns_with_variable_size_2() {
+    assert_warning!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<len, _:size(len)-unit(3)>> -> 1
+    <<len, _:size(len)-unit(2), 1:size(len)>> -> 2
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_overlapping_patterns_with_variable_size_not_redundant() {
+    assert_no_warnings!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<len, 1:size(len)-unit(3)>> -> 1
+    <<len, _:size(len)-unit(2), 1:size(len)>> -> 2
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_patterns_with_different_length_with_same_name_are_not_redundant() {
+    assert_no_warnings!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  let len = 10
+  case bit_array {
+    <<_, _:size(len)-unit(3)>> -> 1
+    // Down here len is not the same as the len above, so the branch below is
+    // not redundant!
+    <<len, _:size(len)-unit(3)>> -> 2
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_patterns_with_different_length_with_same_name_are_not_redundant_1() {
+    assert_no_warnings!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  let len = 10
+  case bit_array {
+    <<len, _:size(len)-unit(3)>> -> 1
+    // Down here len is not the same as the len above, so the branch below is
+    // not redundant!
+    <<_, _:size(len)-unit(3)>> -> 2
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn bit_array_patterns_with_different_length_with_same_name_are_not_redundant_2() {
+    assert_no_warnings!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<_, len, _:size(len)>> -> 1
+    // Down here len is not the same as the len above, so the branch below is
+    // not redundant!
+    <<len, _, _:size(len)>> -> 2
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn same_catch_all_bytes_are_redundant() {
+    assert_warning!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<_:bytes>> -> <<>>
+    <<a:bytes>> -> a
+    _ -> <<>>
+  }
+}"#
+    );
+}
+
+#[test]
+fn different_catch_all_bytes_are_not_redundant() {
+    assert_no_warnings!(
+        r#"pub fn main() {
+  let bit_array = <<>>
+  case bit_array {
+    <<_, _:bytes>> -> <<>>
+    <<_:bytes>> -> <<>>
+    _ -> <<>>
+  }
+}"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2616
+#[test]
+fn duplicated_alternative_patterns() {
+    assert_warning!(
+        "
+pub fn main() {
+  let x = 1
+  case x {
+    2 | 2 -> 2
+    _ -> panic
+  }
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2616
+#[test]
+fn duplicated_pattern_in_alternative() {
+    assert_warning!(
+        "
+pub fn main() {
+  let x = 1
+  case x {
+    2 -> x
+    1 | 2 -> x - 4
+    _ -> panic
+  }
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2616
+#[test]
+fn duplicated_pattern_with_multiple_alternatives() {
+    assert_warning!(
+        "
+pub fn main() {
+  let x = 1
+  case x {
+    1 -> 1
+    3 -> 3
+    5 -> 5
+    1 | 2 | 3 | 4 | 5 -> x - 1
+    _ -> panic
+  }
+}
+"
+    );
+}
+
+#[test]
+fn unreachable_multi_pattern() {
+    assert_warning!(
+        "
+pub fn main() {
+  let x = 1
+  let y = 2
+  case x, y {
+    1, 2 -> True
+    1, 2 -> False
+    _, _ -> panic
+  }
+}
+"
+    );
+}
+
+#[test]
+fn unreachable_alternative_multi_pattern() {
+    assert_warning!(
+        "
+pub fn main() {
+  let x = 1
+  let y = 2
+  case x, y {
+    1, 2 -> True
+    3, 4 | 1, 2 -> False
+    _, _ -> panic
+  }
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4586
+#[test]
+fn compiler_does_not_crash_when_defining_duplicate_alternative_variables() {
+    assert_error!(
+        "
+case todo {
+  #(a, b) | #(a, a as b) -> todo
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4626
+#[test]
+fn correct_missing_patterns_for_opaque_type() {
+    assert_module_error!(
+        (
+            "mod",
+            "pub opaque type Wibble { Wibble(Int) Wobble(String) }"
+        ),
+        "
+import mod
+
+pub fn main(w: mod.Wibble) {
+  case w {}
+}
+"
+    );
+}
+
+#[test]
+fn correct_missing_patterns_for_opaque_type_in_definition_module() {
+    assert_module_error!(
+        "
+pub opaque type Wibble { Wibble(Int) Wobble(String) }
+
+pub fn main(w: Wibble) {
+  case w {}
+}
+"
+    );
+}
