@@ -1,4 +1,4 @@
-use crate::{assert_infer_with_module, assert_module_error, assert_module_infer};
+use crate::{assert_module_error, assert_module_infer};
 
 #[test]
 fn nested_record_access() {
@@ -48,7 +48,7 @@ pub fn a(a: String) {
 
 #[test]
 fn qualified_record() {
-    assert_infer_with_module!(
+    assert_module_infer!(
         ("wibble", "pub type Wibble { Wibble Wobble }"),
         "
 import wibble
@@ -66,7 +66,7 @@ pub fn main(wibble: wibble.Wibble) {
 
 #[test]
 fn qualified_record_with_arguments() {
-    assert_infer_with_module!(
+    assert_module_infer!(
         (
             "wibble",
             "pub type Wibble { Wibble(Int) Wobble(Int, Float) }"
